/* gtkaboutdialog.c
 *
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkaboutdialog.h"

#include "gtkmdkintl.h"


static void gtk_about_dialog_class_init(GtkAboutDialogClass *klass);
static void gtk_about_dialog_init(GtkAboutDialog *about_dialog);

static gboolean gtk_about_dialog_expose(GtkAboutDialog *about_dialog);
static int gtk_about_dialog_timer(GtkAboutDialog *about_dialog);

static void gtk_about_dialog_unmap(GtkWidget *wid);
static void gtk_about_dialog_destroy(GtkObject *obj);

static GtkWindowClass *about_dialog_parent_class = NULL;

/*************************************************************************/
GtkType gtk_about_dialog_get_type(void)
{
  static GtkType about_dialog_type = 0;

  if (!about_dialog_type)
    {
      static const GtkTypeInfo about_dialog_info =
      {
        "GtkAboutDialog",
        sizeof(GtkAboutDialog),
        sizeof(GtkAboutDialogClass),
        (GtkClassInitFunc)gtk_about_dialog_class_init,
        (GtkObjectInitFunc)gtk_about_dialog_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      about_dialog_type = gtk_type_unique(GTK_TYPE_WINDOW, &about_dialog_info);
    }

  return about_dialog_type;
}

/*************************************************************************/
static void gtk_about_dialog_class_init(GtkAboutDialogClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
      
  about_dialog_parent_class = gtk_type_class(GTK_TYPE_WINDOW);
  object_class->destroy = gtk_about_dialog_destroy;
  widget_class->unmap = gtk_about_dialog_unmap;
}

/*************************************************************************/
static void gtk_about_dialog_init(GtkAboutDialog *about_dialog)
{
  GtkWidget *vbox, *alignment, *frame, *separator, *button;

  about_dialog->ContribList = NULL;
  about_dialog->NbContribs = 0;
  about_dialog->ScrollPixmap = NULL;
  about_dialog->ScrollState =0;
  about_dialog->offset = 0;
  about_dialog->CurScrollIndex = 0;
  about_dialog->timer = 0;

  gtk_widget_set(GTK_WIDGET(about_dialog),
                 "GtkWindow::type", GTK_WINDOW_DIALOG, NULL);
  gtk_window_set_policy(GTK_WINDOW(about_dialog), FALSE, FALSE, FALSE);
  gtk_window_position(GTK_WINDOW(about_dialog), GTK_WIN_POS_CENTER_ALWAYS);
          
  vbox = gtk_vbox_new(FALSE, 10);
  gtk_widget_show(vbox);
  gtk_container_add(GTK_CONTAINER(about_dialog), vbox);
  
  about_dialog->AppName = gtk_label_new("");
  gtk_misc_set_padding(GTK_MISC(about_dialog->AppName), 10, 10);
  gtk_box_pack_start_defaults(GTK_BOX(vbox), about_dialog->AppName);
  gtk_widget_show(about_dialog->AppName);
  
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
  gtk_widget_show(separator);
  alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
  gtk_container_set_border_width(GTK_CONTAINER(alignment), 5);
  gtk_widget_show(alignment);
  gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, TRUE, 0);

  frame = gtk_frame_new(NULL);
  gtk_widget_show(frame);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
  gtk_container_border_width(GTK_CONTAINER(frame), 0);
  gtk_container_add(GTK_CONTAINER(alignment), frame);

  about_dialog->ScrollArea = gtk_drawing_area_new();
  gtk_widget_show(about_dialog->ScrollArea);
  gtk_container_add(GTK_CONTAINER(frame), about_dialog->ScrollArea);
  gtk_widget_realize(about_dialog->ScrollArea);
  gdk_window_set_background(about_dialog->ScrollArea->window,
                            &(about_dialog->ScrollArea->style->white));

  button = gtk_button_new_with_label(_("OK"));
  gtk_box_pack_end(GTK_BOX(vbox), button, FALSE, FALSE, 0);
  gtk_signal_connect_object(GTK_OBJECT(button), 
                            "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(about_dialog));
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  gtk_widget_show(button);
}

/*************************************************************************/
GtkWidget *gtk_about_dialog_new(char *AppName, char **ContribList)
{
  GtkAboutDialog *about_dialog;
  char **Contributors;
  gint MaxWidth, i;
  GtkStyle *style;

  about_dialog = gtk_type_new(GTK_TYPE_ABOUT_DIALOG);
  style = about_dialog->ScrollArea->parent->style;

  MaxWidth = 0;
  for (Contributors = ContribList ; *Contributors ; Contributors++)
  {
    about_dialog->Widths[about_dialog->NbContribs] = 
      gdk_string_width(style->font, *Contributors);
    MaxWidth = MAX(MaxWidth, about_dialog->Widths[about_dialog->NbContribs]);
    about_dialog->NbContribs++;
  }

  about_dialog->ContribList = g_malloc(about_dialog->NbContribs * sizeof(gchar *));
  for (i=0 ; i<about_dialog->NbContribs ; i++)
    about_dialog->ContribList[i] = g_strdup(ContribList[i]);

  gtk_signal_connect(GTK_OBJECT(about_dialog), "expose_event",
                     (GtkSignalFunc)gtk_about_dialog_expose, NULL);
  gtk_widget_set_events(GTK_WIDGET(about_dialog), GDK_BUTTON_PRESS_MASK);
  gtk_signal_connect(GTK_OBJECT(about_dialog), "button_press_event",
                     (GtkSignalFunc)gtk_widget_destroy, NULL);
     
     
  
  gtk_drawing_area_size(GTK_DRAWING_AREA(about_dialog->ScrollArea),
                        MaxWidth + 10,
                        style->font->ascent + style->font->descent);
/*  gtk_widget_set_events(ScrollArea, GDK_BUTTON_PRESS_MASK);*/

  if (AppName)
  {
    gtk_label_set(GTK_LABEL(about_dialog->AppName), AppName);
  }
  return GTK_WIDGET(about_dialog);
}

/*************************************************************************/
static gboolean gtk_about_dialog_expose(GtkAboutDialog *about_dialog)
{
  if (!about_dialog->timer)
  {
    gtk_about_dialog_timer(about_dialog);
    about_dialog->timer = gtk_timeout_add(75,
                                          (GtkFunction)gtk_about_dialog_timer,
                                          about_dialog);
  }
  return FALSE;
}

/*************************************************************************/
static int gtk_about_dialog_timer(GtkAboutDialog *about_dialog)
{
  gint return_val;

  return_val = TRUE;
  if (!about_dialog->ScrollPixmap)
    about_dialog->ScrollPixmap = 
      gdk_pixmap_new(about_dialog->ScrollArea->window,
                     about_dialog->ScrollArea->allocation.width,
                     about_dialog->ScrollArea->allocation.height,
                     -1);
  switch (about_dialog->ScrollState)
  {
    case 1:
      about_dialog->ScrollState = 2;
      about_dialog->timer = gtk_timeout_add(700,
                                            (GtkFunction)gtk_about_dialog_timer,
                                            about_dialog);
      return_val = FALSE;
      break;
    case 2:
      about_dialog->ScrollState = 3;
      about_dialog->timer = gtk_timeout_add(75,
                                            (GtkFunction)gtk_about_dialog_timer,
                                            about_dialog);
      return_val = FALSE;
      break;
  }

  if (about_dialog->offset > 
           (about_dialog->Widths[about_dialog->CurScrollIndex] +
            about_dialog->ScrollArea->allocation.width))
  {
    about_dialog->ScrollState = 0;
    about_dialog->CurScrollIndex++;
    if (about_dialog->CurScrollIndex == about_dialog->NbContribs)
      about_dialog->CurScrollIndex = 0;
    about_dialog->offset = 0;
  }
  gdk_draw_rectangle(about_dialog->ScrollPixmap,
                     about_dialog->ScrollArea->style->white_gc,
                     TRUE, 0, 0,
                     about_dialog->ScrollArea->allocation.width,
                     about_dialog->ScrollArea->allocation.height);
  gdk_draw_string(about_dialog->ScrollPixmap,
                  about_dialog->ScrollArea->style->font,
                  about_dialog->ScrollArea->style->black_gc,
                  about_dialog->ScrollArea->allocation.width - 
                         about_dialog->offset,
                  about_dialog->ScrollArea->style->font->ascent,
                  about_dialog->ContribList[about_dialog->CurScrollIndex]);
  gdk_draw_pixmap(about_dialog->ScrollArea->window,
                  about_dialog->ScrollArea->style->black_gc,
                  about_dialog->ScrollPixmap, 0, 0, 0, 0,
                  about_dialog->ScrollArea->allocation.width,
                  about_dialog->ScrollArea->allocation.height);
  about_dialog->offset += 15;
  if (about_dialog->ScrollState == 0)
  {
    if (about_dialog->offset > 
          ((about_dialog->ScrollArea->allocation.width + 
            about_dialog->Widths[about_dialog->CurScrollIndex]) / 2))
    {
      about_dialog->ScrollState = 1;
      about_dialog->offset = 
         (about_dialog->ScrollArea->allocation.width + 
          about_dialog->Widths[about_dialog->CurScrollIndex]) / 2;
    }
  }
  return return_val;
}

/*************************************************************************/
static void gtk_about_dialog_unmap(GtkWidget *wid)
{
  GtkAboutDialog *about_dialog;
  g_return_if_fail(wid != NULL);
  g_return_if_fail(GTK_IS_ABOUT_DIALOG(wid));
  about_dialog = GTK_ABOUT_DIALOG(wid);

  if (about_dialog->timer)
  {
    gtk_timeout_remove(about_dialog->timer);
    about_dialog->timer = 0;
  }
    
  if (GTK_WIDGET_CLASS(about_dialog_parent_class)->unmap)
    (* (GTK_WIDGET_CLASS(about_dialog_parent_class)->unmap))(wid);
}

/*************************************************************************/
static void gtk_about_dialog_destroy(GtkObject *obj)
{
  GtkAboutDialog *about_dialog;
  int i;

  g_return_if_fail(obj != NULL);
  g_return_if_fail(GTK_IS_ABOUT_DIALOG(obj));
  about_dialog = GTK_ABOUT_DIALOG(obj);

  /* free the Contributor array */
  for (i=0 ; i<about_dialog->NbContribs ; i++)
    g_free(about_dialog->ContribList[i]);
  g_free(about_dialog->ContribList);
  /* Stop the timer */
  if (about_dialog->timer)
  {
    gtk_timeout_remove(about_dialog->timer);
    about_dialog->timer = 0;
  }
  /* and call the parent destructor */
  if (GTK_OBJECT_CLASS(about_dialog_parent_class)->destroy)
    (* (GTK_OBJECT_CLASS(about_dialog_parent_class)->destroy))(obj);
}

