/* gtkdirselect.c
 *
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkdirselect.h"

#include "gtkmdkintl.h"

enum
{
  DIR_CHANGED,
  LAST_SIGNAL
};

enum
{
  ARG_0,
  ARG_LABEL,
  ARG_DIRECTORY,
  ARG_FS_TITLE
};

static void gtk_dir_select_class_init(GtkDirSelectClass *klass);
static void gtk_dir_select_init(GtkDirSelect *dirselect);
static void gtk_dir_select_set_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id);
static void gtk_dir_select_get_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id);
static void gtk_dir_select_finalize(GtkObject *object);
static void gtk_dir_select_change_source_dir(GtkWidget *dir_select);
static void gtk_dir_select_update_source_dir(GtkWidget *Button,
                                             GtkDirSelect *dir_select);

static GtkHBoxClass *dir_select_parent_class = NULL;
static guint dir_select_signals[LAST_SIGNAL] = {0};

/*************************************************************************/
GtkType gtk_dir_select_get_type(void)
{
  static GtkType dir_select_type = 0;

  if (!dir_select_type)
    {
      static const GtkTypeInfo dir_select_info =
      {
        "GtkDirSelect",
        sizeof(GtkDirSelect),
        sizeof(GtkDirSelectClass),
        (GtkClassInitFunc)gtk_dir_select_class_init,
        (GtkObjectInitFunc)gtk_dir_select_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      dir_select_type = gtk_type_unique (GTK_TYPE_HBOX, &dir_select_info);
    }

  return dir_select_type;
}

/*************************************************************************/
static void gtk_dir_select_class_init(GtkDirSelectClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  GtkContainerClass *container_class;

  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;
  container_class = (GtkContainerClass*) klass;

  dir_select_parent_class = gtk_type_class(GTK_TYPE_HBOX);
  
  gtk_object_add_arg_type("GtkDirSelect::label", GTK_TYPE_STRING,
                          GTK_ARG_READWRITE, ARG_LABEL);
  gtk_object_add_arg_type("GtkDirSelect::directory", GTK_TYPE_STRING,
                          GTK_ARG_READWRITE, ARG_DIRECTORY);
  gtk_object_add_arg_type("GtkDirSelect::FS_title", GTK_TYPE_STRING,
                          GTK_ARG_READWRITE, ARG_FS_TITLE);

  dir_select_signals[DIR_CHANGED] = gtk_signal_new(
      "dir_changed", GTK_RUN_FIRST, object_class->type,
      GTK_SIGNAL_OFFSET(GtkDirSelectClass, dir_changed),
      gtk_marshal_NONE__POINTER, GTK_TYPE_NONE, 1, GTK_TYPE_STRING);

  gtk_object_class_add_signals(object_class, dir_select_signals, LAST_SIGNAL);

  object_class->set_arg = gtk_dir_select_set_arg;
  object_class->get_arg = gtk_dir_select_get_arg;
  object_class->finalize = gtk_dir_select_finalize;
}

/*************************************************************************/
static void gtk_dir_select_init(GtkDirSelect *dirselect)
{
  dirselect->label = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(dirselect), dirselect->label, FALSE, FALSE, 0);
  gtk_widget_show(dirselect->label);
  dirselect->entry = gtk_entry_new();
  gtk_container_add(GTK_CONTAINER(dirselect), dirselect->entry);
  gtk_widget_show(dirselect->entry);
  dirselect->button = gtk_button_new_with_label(_(" ... "));
  gtk_box_pack_start(GTK_BOX(dirselect), dirselect->button, FALSE, FALSE, 0);
  gtk_signal_connect_object(GTK_OBJECT(dirselect->button), "clicked",
                            (GtkSignalFunc)gtk_dir_select_change_source_dir,
                            GTK_OBJECT(dirselect));
    
  gtk_widget_show(dirselect->button);
  dirselect->directory = NULL;
  dirselect->FStitle = NULL;
}

/*************************************************************************/
static void gtk_dir_select_set_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id)
{
  GtkDirSelect *dir_select = GTK_DIR_SELECT(object);
  
  switch (arg_id)
  {
    case ARG_LABEL:
      gtk_label_set_text(GTK_LABEL(dir_select->label), GTK_VALUE_STRING(*arg));
      break;
    case ARG_DIRECTORY:
/* FIXME      gtk_dir_select_set_directory(dir_select, GTK_VALUE_STRING(*arg));*/
      break;
    case ARG_FS_TITLE:
      if (dir_select->FStitle)
        g_free(dir_select->FStitle);
      dir_select->FStitle = g_strdup(GTK_VALUE_STRING(*arg));
      break;
  }
}

/*************************************************************************/
static void gtk_dir_select_get_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id)
{
  GtkDirSelect *dir_select;
  
  dir_select = GTK_DIR_SELECT(object);
  
  switch (arg_id)
  {
    case ARG_LABEL:
      GTK_VALUE_STRING(*arg) = g_strdup(GTK_LABEL(dir_select->label)->label);
      break;
    case ARG_DIRECTORY:
      GTK_VALUE_STRING(*arg) = g_strdup(dir_select->directory);
      break;
    case ARG_FS_TITLE:
      GTK_VALUE_STRING(*arg) = g_strdup(dir_select->FStitle);
      break;
  }
}

/*************************************************************************/
GtkWidget *gtk_dir_select_new(gchar *label,
                              gchar *directory,
                              gchar *FStitle)
{
  GtkDirSelect *dir_select;

  dir_select = gtk_type_new(GTK_TYPE_DIR_SELECT);

  if (label)
    gtk_label_set_text(GTK_LABEL(dir_select->label), label);

  if (directory)
  {
    dir_select->directory = g_strdup(directory);
    gtk_entry_set_text(GTK_ENTRY(dir_select->entry), dir_select->directory);
  }

  if (FStitle)
    dir_select->FStitle = g_strdup(FStitle);

  return GTK_WIDGET(dir_select);
}

/*************************************************************************/
static void gtk_dir_select_finalize(GtkObject *object)
{
  GtkDirSelect *dir_select;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_DIR_SELECT(object));

  dir_select = GTK_DIR_SELECT(object);

  if (dir_select->directory != NULL)
    g_free(dir_select->directory);

  if (dir_select->FStitle != NULL)
    g_free(dir_select->FStitle);

  (*GTK_OBJECT_CLASS (dir_select_parent_class)->finalize) (object);
}

/*************************************************************************/
static void gtk_dir_select_dir_changed(GtkDirSelect *dir_select)
{
  gtk_signal_emit(GTK_OBJECT(dir_select),
                  dir_select_signals[DIR_CHANGED],
                  dir_select->directory);
}

/*************************************************************************/
static void gtk_dir_select_update_source_dir(GtkWidget *Button,
                                             GtkDirSelect *dir_select)
{
  GtkFileSelection *FS;

  FS = GTK_FILE_SELECTION(gtk_widget_get_toplevel(Button));
  if (dir_select->directory)
    g_free(dir_select->directory);
  dir_select->directory = g_strdup(gtk_file_selection_get_filename(FS));
  gtk_entry_set_text(GTK_ENTRY(dir_select->entry), dir_select->directory);
  gtk_dir_select_dir_changed(dir_select);
}

/*************************************************************************/
static void gtk_dir_select_change_source_dir(GtkWidget *widget)
{
  GtkFileSelection *FS;
  GtkDirSelect *dir_select = GTK_DIR_SELECT(widget);
  
  FS = GTK_FILE_SELECTION(gtk_file_selection_new(dir_select->FStitle));
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(FS->dir_list->parent),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_widget_hide(FS->file_list->parent);
  gtk_widget_hide(FS->selection_entry);
  gtk_widget_hide(FS->selection_text);
  if (dir_select->directory)
    gtk_file_selection_set_filename(FS, dir_select->directory);
  gtk_file_selection_hide_fileop_buttons(FS);
  gtk_window_set_modal(GTK_WINDOW(FS), TRUE);
  gtk_signal_connect_object(GTK_OBJECT(FS->cancel_button),
                            "clicked", (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(FS));
  gtk_signal_connect(GTK_OBJECT(FS->ok_button),
                     "clicked", (GtkSignalFunc)gtk_dir_select_update_source_dir,
                     GTK_OBJECT(dir_select));
  gtk_signal_connect_object(GTK_OBJECT(FS->ok_button),
                            "clicked", (GtkSignalFunc)gtk_widget_destroy,
                            GTK_OBJECT(FS));
  gtk_widget_show(GTK_WIDGET(FS));
}

/*************************************************************************/
void gtk_dir_select_set_dir(GtkDirSelect *dir_select, gchar *new_dir)
{
  g_return_if_fail(dir_select != NULL);
  g_return_if_fail(GTK_IS_DIR_SELECT(dir_select));
 
  if (dir_select->directory)
    g_free(dir_select->directory);
  dir_select->directory = g_strdup(new_dir);
  gtk_entry_set_text(GTK_ENTRY(dir_select->entry), dir_select->directory);
  gtk_dir_select_dir_changed(dir_select);
}

/*************************************************************************/
gchar *gtk_dir_select_get_dir(GtkDirSelect *dir_select)
{
  g_return_if_fail(dir_select != NULL);
  g_return_if_fail(GTK_IS_DIR_SELECT(dir_select));
  
  return dir_select->directory;
}

