/* gtkframedcombo.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkframedcombo.h"

#include "gtkmdkintl.h"

static void gtk_framed_combo_class_init(GtkFramedComboClass *klass);
static void gtk_framed_combo_init(GtkFramedCombo *framed_combo);
static void gtk_framed_combo_finalize(GtkObject *object);

static GtkFrameClass *framed_combo_parent_class = NULL;

/*************************************************************************/
GtkType gtk_framed_combo_get_type(void)
{
  static GtkType framed_combo_type = 0;

  if (!framed_combo_type)
  {
    static const GtkTypeInfo framed_combo_info =
    {
      "GtkFramedCombo",
      sizeof(GtkFramedCombo),
      sizeof(GtkFramedComboClass),
      (GtkClassInitFunc)gtk_framed_combo_class_init,
      (GtkObjectInitFunc)gtk_framed_combo_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
      (GtkClassInitFunc) NULL,
    };
    framed_combo_type = gtk_type_unique(GTK_TYPE_FRAME, &framed_combo_info);
  }
  return framed_combo_type;
}

/*************************************************************************/
static void gtk_framed_combo_class_init(GtkFramedComboClass *klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) klass;

  framed_combo_parent_class = gtk_type_class(GTK_TYPE_FRAME);

  object_class->finalize = gtk_framed_combo_finalize;
}

/*************************************************************************/
static void gtk_framed_combo_init(GtkFramedCombo *framed_combo)
{
  GtkWidget *vbox;

  vbox = gtk_vbox_new(FALSE, 3);
  gtk_container_add(GTK_CONTAINER(framed_combo), vbox);
  framed_combo->Label = gtk_label_new("");
  gtk_box_pack_start_defaults(GTK_BOX(vbox), framed_combo->Label);
  framed_combo->Combo = gtk_combo_new();
  gtk_box_pack_start_defaults(GTK_BOX(vbox), framed_combo->Combo);
  framed_combo->entry = GTK_COMBO(framed_combo->Combo)->entry;
  framed_combo->list = GTK_COMBO(framed_combo->Combo)->list;
}

/*************************************************************************/
GtkWidget *gtk_framed_combo_new(gchar *Label)
{
  GtkFramedCombo *framed_combo;

  framed_combo = gtk_type_new(GTK_TYPE_FRAMED_COMBO);

  gtk_label_set_text(GTK_LABEL(framed_combo->Label), Label);

  return GTK_WIDGET(framed_combo);
}

/*************************************************************************/
static void gtk_framed_combo_finalize(GtkObject *object)
{
  (*GTK_OBJECT_CLASS(framed_combo_parent_class)->finalize)(object);
}

/* the text in the entry must be or not be in the list */
void gtk_framed_combo_set_value_in_list(GtkFramedCombo *FramedCombo,
                                        gint val,
                                        gint ok_if_empty)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_value_in_list(GTK_COMBO(FramedCombo->Combo), val,
                              ok_if_empty);
}

/* set/unset arrows working for changing the value (can be annoying */
void gtk_framed_combo_set_use_arrows(GtkFramedCombo *FramedCombo,
                                     gint val)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_use_arrows(GTK_COMBO(FramedCombo->Combo), val);
}

/* up/down arrows change value if current value not in list */
void gtk_framed_combo_set_use_arrows_always(GtkFramedCombo *FramedCombo,
                                            gint val)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_use_arrows_always(GTK_COMBO(FramedCombo->Combo), val);
}

/* perform case-sensitive compares */
void gtk_framed_combo_set_case_sensitive(GtkFramedCombo *FramedCombo,
                                         gint val)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_case_sensitive(GTK_COMBO(FramedCombo->Combo), val);
}

/* call this function on an item if it isn't a label or you
 * want it to have a different value to be displayed in the entry */
void gtk_framed_combo_set_item_string(GtkFramedCombo *FramedCombo,
                                      GtkItem *item,
                                      const gchar *item_value)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_item_string(GTK_COMBO(FramedCombo->Combo), item, item_value);
}

/* simple interface */
void gtk_framed_combo_set_popdown_strings(GtkFramedCombo *FramedCombo,
                                          GList *strings)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_set_popdown_strings(GTK_COMBO(FramedCombo->Combo), strings);
}

/*************************************************************************/
void gtk_framed_combo_disable_activate(GtkFramedCombo *FramedCombo)
{
  g_return_if_fail(FramedCombo != NULL);
  g_return_if_fail(GTK_IS_FRAMED_COMBO(FramedCombo));

  gtk_combo_disable_activate(GTK_COMBO(FramedCombo->Combo));
}

