/* gtkhelpbutton.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include "gtkhelpbutton.h"

#include "gtkmdkintl.h"

enum
{
  ARG_0,
  ARG_HELP_FILE
};

static void gtk_help_button_class_init(GtkHelpButtonClass *klass);
static void gtk_help_button_init(GtkHelpButton *help_button);
static void gtk_help_button_set_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id);
static void gtk_help_button_get_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id);
static void gtk_help_button_finalize(GtkObject *object);
static void gtk_help_button_show_help(GtkHelpButton *help_button);

static GtkButtonClass *help_button_parent_class = NULL;

/*************************************************************************/
GtkType gtk_help_button_get_type(void)
{
  static GtkType help_button_type = 0;

  if (!help_button_type)
    {
      static const GtkTypeInfo help_button_info =
      {
        "GtkHelpButton",
        sizeof(GtkHelpButton),
        sizeof(GtkHelpButtonClass),
        (GtkClassInitFunc)gtk_help_button_class_init,
        (GtkObjectInitFunc)gtk_help_button_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      help_button_type = gtk_type_unique(GTK_TYPE_BUTTON, &help_button_info);
    }

  return help_button_type;
}

/*************************************************************************/
static void gtk_help_button_class_init(GtkHelpButtonClass *klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) klass;

  help_button_parent_class = gtk_type_class(GTK_TYPE_BUTTON);
  
  gtk_object_add_arg_type("GtkHelpButton::HelpFile", GTK_TYPE_STRING,
                          GTK_ARG_READWRITE, ARG_HELP_FILE);

  object_class->set_arg = gtk_help_button_set_arg;
  object_class->get_arg = gtk_help_button_get_arg;
  object_class->finalize = gtk_help_button_finalize;
}

/*************************************************************************/
static void gtk_help_button_init(GtkHelpButton *help_button)
{
  GtkWidget *Label;

  Label = gtk_label_new(_(" Help "));
  gtk_container_add(GTK_CONTAINER(help_button), Label);
  gtk_signal_connect(GTK_OBJECT(help_button), "clicked",
                     (GtkSignalFunc)gtk_help_button_show_help, NULL);
  gtk_widget_show(Label);
}

/*************************************************************************/
static void gtk_help_button_set_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id)
{
  GtkHelpButton *help_button = GTK_HELP_BUTTON(object);
  
  switch (arg_id)
  {
    case ARG_HELP_FILE:
      if (help_button->HelpFile)
        g_free(help_button->HelpFile);
      help_button->HelpFile = g_strdup(GTK_VALUE_STRING(*arg));
      break;
  }
}

/*************************************************************************/
static void gtk_help_button_get_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id)
{
  GtkHelpButton *help_button;
  
  help_button = GTK_HELP_BUTTON(object);
  
  switch (arg_id)
  {
    case ARG_HELP_FILE:
      GTK_VALUE_STRING(*arg) = g_strdup(help_button->HelpFile);
      break;
  }
}

/*************************************************************************/
GtkWidget *gtk_help_button_new(gchar *HelpFile)
{
  GtkHelpButton *help_button;

  help_button = gtk_type_new(GTK_TYPE_HELP_BUTTON);

  if (HelpFile)
    help_button->HelpFile = g_strdup(HelpFile);

  return GTK_WIDGET(help_button);
}

/*************************************************************************/
static void gtk_help_button_finalize(GtkObject *object)
{
  GtkHelpButton *help_button;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_HELP_BUTTON(object));

  help_button = GTK_HELP_BUTTON(object);

  if (help_button->HelpFile != NULL)
    g_free(help_button->HelpFile);

  if (help_button->HelpDialog)
    gtk_widget_destroy(help_button->HelpDialog);

  (*GTK_OBJECT_CLASS(help_button_parent_class)->finalize)(object);
}

/*************************************************************************/
static void gtk_help_button_show_help(GtkHelpButton *help_button)
{
  GtkWidget *Button;
  GtkWidget *ScrolledWindow;
  GtkWidget *Text;
  FILE *HelpFile;

  if (help_button->HelpDialog)
  {
    if (!GTK_WIDGET_VISIBLE(help_button->HelpDialog))
    {
      gtk_widget_show(help_button->HelpDialog);
    }
    else
    {
      gdk_window_raise(help_button->HelpDialog->window);
    }
    return;
  }

  help_button->HelpDialog = gtk_dialog_new();
/*  gtk_window_set_title(GTK_WINDOW(help_button->HelpDialog), "Help");*/
  gtk_window_set_policy(GTK_WINDOW(help_button->HelpDialog), TRUE, TRUE, FALSE);
  gtk_widget_set_usize(help_button->HelpDialog, 500, 500);

  gtk_signal_connect(GTK_OBJECT(help_button->HelpDialog), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                     &(help_button->HelpDialog));
  /* The Close Button */
  Button = gtk_button_new_with_label(_("Close"));
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(help_button->HelpDialog)->action_area),
                              Button);
  gtk_signal_connect_object(GTK_OBJECT(Button), "clicked",
                            GTK_SIGNAL_FUNC(gtk_widget_destroy),
                            GTK_OBJECT(help_button->HelpDialog));
  gtk_widget_grab_default(Button);
  /* The Scrolled window */
  ScrolledWindow = gtk_scrolled_window_new(NULL, NULL);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(help_button->HelpDialog)->vbox),
                              ScrolledWindow);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ScrolledWindow),
                                 GTK_POLICY_NEVER,
                                 GTK_POLICY_AUTOMATIC);
  /* The text widget */
  Text = gtk_text_new(NULL, NULL);
  gtk_text_set_editable(GTK_TEXT(Text), FALSE);
  gtk_container_add(GTK_CONTAINER(ScrolledWindow), Text);
  gtk_widget_grab_focus(Text);

  gtk_text_freeze(GTK_TEXT(Text));
  HelpFile = fopen(help_button->HelpFile, "r");
  if (HelpFile)
  {
    char *buffer;
    int nbytes_read, nbytes_alloc;
    nbytes_read = 0;
    nbytes_alloc = 1024;
    buffer = g_new(char, nbytes_alloc);
    while (1)
    {
      int len;
      if (nbytes_alloc < nbytes_read+1024)
      {
        nbytes_alloc *= 2;
        buffer = g_realloc(buffer, nbytes_alloc);
      }
      len = fread(buffer+nbytes_read, 1, 1024, HelpFile);
      nbytes_read += len;
      if (len < 1024)
        break;
    }
    gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, buffer, nbytes_read);
    g_free(buffer);
    fclose(HelpFile);
  } else
  {
    char *buffer = _("The help file was not found\n");
    gtk_text_insert(GTK_TEXT(Text), NULL, NULL, NULL, buffer, strlen(buffer));
  }
  gtk_text_thaw(GTK_TEXT(Text));
  gtk_widget_show_all(help_button->HelpDialog);
}

