/* gtkmdkdialog.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gtkmdkdialog.h"

#include "gtkmdkintl.h"

#include "ask.xpm"
#include "bang.xpm"
#include "stop.xpm"
enum
{
  ARG_0,
  ARG_ICON
};

enum
{
  CLICKED,
  CLOSE,
  LAST_SIGNAL
};

typedef void (*GtkMdkDialogSignal1)(GtkObject *object,
                                    gint arg1,
                                    gpointer data);

typedef gboolean (*GtkMdkDialogSignal2)(GtkObject *object,
                                        gpointer data);

static void gtk_mdk_dialog_marshal_signal_1(GtkObject *object,
                                            GtkSignalFunc func,
                                            gpointer func_data,
                                            GtkArg *args);
static void gtk_mdk_dialog_marshal_signal_2(GtkObject *object,
                                            GtkSignalFunc func,
                                            gpointer func_data,
                                            GtkArg *args);

static void gtk_mdk_dialog_class_init(GtkMdkDialogClass *klass);
static void gtk_mdk_dialog_init(GtkMdkDialog *mdk_dialog);
static void gtk_mdk_dialog_set_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id);
static void gtk_mdk_dialog_get_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id);
static void gtk_mdk_dialog_destroy(GtkObject *dialog);
static gint gtk_mdk_dialog_key_pressed(GtkWidget *d, GdkEventKey *e);
static gint gtk_mdk_dialog_delete_event(GtkWidget *d, GdkEventAny *e);
static void gtk_mdk_dialog_show(GtkWidget *d);
static void gtk_mdk_dialog_button_clicked(GtkWidget *button, GtkWidget *dialog);

static GtkDialogClass *mdk_dialog_parent_class = NULL;
static gint gtk_mdk_dialog_signals[LAST_SIGNAL] = { 0, 0 };


/*************************************************************************/
GtkType gtk_mdk_dialog_get_type(void)
{
  static GtkType mdk_dialog_type = 0;

  if (!mdk_dialog_type)
    {
      static const GtkTypeInfo mdk_dialog_info =
      {
        "GtkMdkDialog",
        sizeof(GtkMdkDialog),
        sizeof(GtkMdkDialogClass),
        (GtkClassInitFunc)gtk_mdk_dialog_class_init,
        (GtkObjectInitFunc)gtk_mdk_dialog_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL
      };

      mdk_dialog_type = gtk_type_unique(GTK_TYPE_DIALOG, &mdk_dialog_info);
    }

  return mdk_dialog_type;
}

/*************************************************************************/
static void gtk_mdk_dialog_class_init(GtkMdkDialogClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*)klass;
  widget_class = (GtkWidgetClass*) klass;

  mdk_dialog_parent_class = gtk_type_class(GTK_TYPE_DIALOG);
  
  gtk_object_add_arg_type("GtkMdkDialog::Icon", GTK_TYPE_STRING,
                          GTK_ARG_READWRITE, ARG_ICON);

  gtk_mdk_dialog_signals[CLOSE] = gtk_signal_new("close",
                                  GTK_RUN_LAST,
                                  object_class->type,
                                  GTK_SIGNAL_OFFSET(GtkMdkDialogClass, close),
                                  gtk_mdk_dialog_marshal_signal_2,
                                  GTK_TYPE_INT, 0);

  gtk_mdk_dialog_signals[CLICKED] = gtk_signal_new("clicked",
                                    GTK_RUN_LAST,
                                    object_class->type,
                                    GTK_SIGNAL_OFFSET(GtkMdkDialogClass, clicked),
                                    gtk_mdk_dialog_marshal_signal_1,
                                    GTK_TYPE_NONE, 1, GTK_TYPE_INT);

  gtk_object_class_add_signals(object_class, gtk_mdk_dialog_signals, LAST_SIGNAL);

  klass->clicked = NULL;
  klass->close = NULL;
 
  object_class->set_arg = gtk_mdk_dialog_set_arg;
  object_class->get_arg = gtk_mdk_dialog_get_arg;
  object_class->destroy = gtk_mdk_dialog_destroy;
  widget_class->key_press_event = gtk_mdk_dialog_key_pressed;
  widget_class->delete_event = gtk_mdk_dialog_delete_event;
  widget_class->show = gtk_mdk_dialog_show;
}

/*************************************************************************/
static void gtk_mdk_dialog_marshal_signal_1(GtkObject *object,
                                            GtkSignalFunc func,
                                            gpointer func_data,
                                            GtkArg *args)
{
  GtkMdkDialogSignal1 rfunc;

  rfunc = (GtkMdkDialogSignal1)func;
  (* rfunc)(object, GTK_VALUE_INT(args[0]), func_data);
}

/*************************************************************************/
static void gtk_mdk_dialog_marshal_signal_2(GtkObject *object,
                                            GtkSignalFunc func,
                                            gpointer func_data,
                                            GtkArg *args)
{
  GtkMdkDialogSignal2 rfunc;
  gint *return_val;

  rfunc = (GtkMdkDialogSignal2) func;
  return_val = GTK_RETLOC_INT(args[0]);
  *return_val = (* rfunc)(object, func_data);
}

/*************************************************************************/
static void gtk_mdk_dialog_init(GtkMdkDialog *mdk_dialog)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));

  mdk_dialog->just_hide = FALSE;
  mdk_dialog->close_on_click = TRUE;

  mdk_dialog->accelerators = gtk_accel_group_new();
  gtk_window_add_accel_group(GTK_WINDOW(mdk_dialog), mdk_dialog->accelerators);
  gtk_window_set_policy (GTK_WINDOW(mdk_dialog), FALSE, FALSE, FALSE);
}

/*************************************************************************/
static void gtk_mdk_dialog_set_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id)
{
  GdkPixmap    *Pix = NULL;
  GdkBitmap    *Mask = NULL;
  GdkColor     Trans;
  GtkMdkDialog *mdk_dialog = GTK_MDK_DIALOG(object);
  
  switch (arg_id)
  {
    case ARG_ICON:
      if (mdk_dialog->icon != NONE_ICON)
	gtk_widget_destroy(mdk_dialog->Pixmap);
      mdk_dialog->icon = GTK_VALUE_ENUM(*arg);
      if (mdk_dialog->icon != NONE_ICON)
      {
        switch (mdk_dialog->icon)
        {
          case STOP_ICON:
            Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                               &Mask,
                                               &Trans,
                                               stop_xpm);
	    break;
          case ASK_ICON:
            Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                               &Mask,
                                               &Trans,
                                               ask_xpm);
            break;
          case BANG_ICON:
            Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                               &Mask,
                                               &Trans,
                                               bang_xpm);
	    break;
	  case NONE_ICON: break;
	}
        mdk_dialog->Pixmap = gtk_pixmap_new(Pix, Mask);
        gtk_misc_set_padding(GTK_MISC(mdk_dialog->Pixmap), 5, 5);
        gdk_pixmap_unref(Pix);
        gdk_pixmap_unref(Mask);
        gtk_box_pack_start_defaults(GTK_BOX(mdk_dialog->hbox),
                                    mdk_dialog->Pixmap);
        gtk_widget_show(mdk_dialog->Pixmap);
      }
      break;
  }
}

/*************************************************************************/
static void gtk_mdk_dialog_get_arg(GtkObject *object,
                                   GtkArg *arg, guint arg_id)
{
  GtkMdkDialog *mdk_dialog = GTK_MDK_DIALOG(object);
  
  switch (arg_id)
  {
    case ARG_ICON:
      GTK_VALUE_ENUM(*arg) = mdk_dialog->icon;
      break;
  }
}

/*************************************************************************/
GtkWidget *gtk_mdk_dialog_new(GtkMdkDialogType type,
                              GtkMdkDialogIcon icon,
                              gchar *title,
                              gchar *text)
{
  GtkMdkDialog *mdk_dialog;
  GdkPixmap    *Pix;
  GdkBitmap    *Mask;
  GdkColor     Trans;

  mdk_dialog = gtk_type_new(GTK_TYPE_MDK_DIALOG);

  mdk_dialog->type = type;
  switch (mdk_dialog->type)
  {
    case GTK_DIALOG_OK:
      mdk_dialog->Buttons[0] = gtk_button_new_with_label(_(" OK "));
      gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->action_area),
                                  mdk_dialog->Buttons[0]);
      gtk_widget_show(mdk_dialog->Buttons[0]);
      gtk_signal_connect_after(GTK_OBJECT(mdk_dialog->Buttons[0]), "clicked",
                               (GtkSignalFunc)gtk_mdk_dialog_button_clicked,
                               mdk_dialog);
      GTK_WIDGET_SET_FLAGS(mdk_dialog->Buttons[0], GTK_CAN_DEFAULT);
      gtk_widget_grab_default(mdk_dialog->Buttons[0]);
      mdk_dialog->NbButtons = 1;
      break;
    case GTK_DIALOG_OK_CANCEL:
      mdk_dialog->Buttons[0] = gtk_button_new_with_label(_(" OK "));
      gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->action_area),
                                  mdk_dialog->Buttons[0]);
      gtk_widget_show(mdk_dialog->Buttons[0]); 
      gtk_signal_connect_after(GTK_OBJECT(mdk_dialog->Buttons[0]), "clicked",
                               (GtkSignalFunc)gtk_mdk_dialog_button_clicked,
                               mdk_dialog);
      GTK_WIDGET_SET_FLAGS(mdk_dialog->Buttons[0], GTK_CAN_DEFAULT);
      gtk_widget_grab_default(mdk_dialog->Buttons[0]);
      mdk_dialog->Buttons[1] = gtk_button_new_with_label(_(" Cancel "));
      gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->action_area),
                                  mdk_dialog->Buttons[1]);
      gtk_widget_show(mdk_dialog->Buttons[1]); 
      gtk_signal_connect_after(GTK_OBJECT(mdk_dialog->Buttons[1]), "clicked",
                               (GtkSignalFunc)gtk_mdk_dialog_button_clicked,
                               mdk_dialog);
      GTK_WIDGET_SET_FLAGS(mdk_dialog->Buttons[1], GTK_CAN_DEFAULT);
      mdk_dialog->NbButtons = 2;
      break;
    case GTK_DIALOG_YES_NO:
      mdk_dialog->Buttons[0] = gtk_button_new_with_label(_(" Yes "));
      gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->action_area),
                                  mdk_dialog->Buttons[0]);
      gtk_widget_show(mdk_dialog->Buttons[0]); 
      gtk_signal_connect_after(GTK_OBJECT(mdk_dialog->Buttons[0]), "clicked",
                               (GtkSignalFunc)gtk_mdk_dialog_button_clicked,
                               mdk_dialog);
      GTK_WIDGET_SET_FLAGS(mdk_dialog->Buttons[0], GTK_CAN_DEFAULT);
      gtk_widget_grab_default(mdk_dialog->Buttons[0]);
      mdk_dialog->Buttons[1] = gtk_button_new_with_label(_(" No "));
      gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->action_area),
                                  mdk_dialog->Buttons[1]);
      gtk_widget_show(mdk_dialog->Buttons[1]); 
      gtk_signal_connect_after(GTK_OBJECT(mdk_dialog->Buttons[1]), "clicked",
                               (GtkSignalFunc)gtk_mdk_dialog_button_clicked,
                               mdk_dialog);
      GTK_WIDGET_SET_FLAGS(mdk_dialog->Buttons[1], GTK_CAN_DEFAULT);
      mdk_dialog->NbButtons = 2;
      break;
  }
  if (title)
  {
    mdk_dialog->title = g_strdup(title);
    gtk_window_set_title(GTK_WINDOW(mdk_dialog), mdk_dialog->title);
  }
  mdk_dialog->hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(mdk_dialog)->vbox), 
                              mdk_dialog->hbox);
  gtk_widget_show(mdk_dialog->hbox);
  gtk_widget_realize(GTK_WIDGET(mdk_dialog));
  switch (icon)
  {
    case STOP_ICON:
      Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                         &Mask,
                                         &Trans,
                                         stop_xpm);
      mdk_dialog->Pixmap = gtk_pixmap_new(Pix, Mask);
      gtk_misc_set_padding(GTK_MISC(mdk_dialog->Pixmap), 5, 5);
      gdk_pixmap_unref(Pix);
      gdk_pixmap_unref(Mask);
      gtk_box_pack_start_defaults(GTK_BOX(mdk_dialog->hbox),
                                  mdk_dialog->Pixmap);
      gtk_widget_show(mdk_dialog->Pixmap);
      break;
    case ASK_ICON:
      Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                         &Mask,
                                         &Trans,
                                         ask_xpm);
      mdk_dialog->Pixmap = gtk_pixmap_new(Pix, Mask);
      gtk_misc_set_padding(GTK_MISC(mdk_dialog->Pixmap), 5, 5);
      gdk_pixmap_unref(Pix);
      gdk_pixmap_unref(Mask);
      gtk_box_pack_start_defaults(GTK_BOX(mdk_dialog->hbox),
                                  mdk_dialog->Pixmap);
      gtk_widget_show(mdk_dialog->Pixmap);
      break;
    case BANG_ICON:
      Pix = gdk_pixmap_create_from_xpm_d(GTK_WIDGET(mdk_dialog)->window,
                                         &Mask,
                                         &Trans,
                                         bang_xpm);
      mdk_dialog->Pixmap = gtk_pixmap_new(Pix, Mask);
      gtk_misc_set_padding(GTK_MISC(mdk_dialog->Pixmap), 5, 5);
      gdk_pixmap_unref(Pix);
      gdk_pixmap_unref(Mask);
      gtk_box_pack_start_defaults(GTK_BOX(mdk_dialog->hbox),
                                  mdk_dialog->Pixmap);
      gtk_widget_show(mdk_dialog->Pixmap);
      break;
    case NONE_ICON:
      break;
  }
  if (text)
  {
    mdk_dialog->text = g_strdup(text);
    mdk_dialog->Label = gtk_label_new(mdk_dialog->text);
    gtk_misc_set_padding(GTK_MISC(mdk_dialog->Label), 5, 5);
    gtk_box_pack_end_defaults(GTK_BOX(mdk_dialog->hbox),
                              mdk_dialog->Label);
    gtk_widget_show(mdk_dialog->Label);
  }

  return GTK_WIDGET(mdk_dialog);
}

/*************************************************************************/
void gtk_mdk_dialog_set_parent(GtkMdkDialog *mdk_dialog,
                               GtkWindow *parent)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(parent != NULL);
  g_return_if_fail(GTK_IS_WINDOW(parent));
  g_return_if_fail(parent != GTK_WINDOW(mdk_dialog));

  gtk_window_set_transient_for(GTK_WINDOW(mdk_dialog), parent);
  /* XXX: May be add more stuff there */
}              

/*************************************************************************/
struct GtkMdkDialogRunInfo
{
  gint button_number;
  gint close_id, clicked_id, destroy_id;
  gboolean destroyed;
};

static void gtk_mdk_dialog_shutdown_run(GtkMdkDialog *mdk_dialog,
                                        struct GtkMdkDialogRunInfo *runinfo)
{
  if (!runinfo->destroyed)
  {
    gtk_signal_disconnect(GTK_OBJECT(mdk_dialog), runinfo->close_id);
    gtk_signal_disconnect(GTK_OBJECT(mdk_dialog), runinfo->clicked_id);
    runinfo->close_id = runinfo->clicked_id = -1;
  }
  gtk_main_quit();
}

/*************************************************************************/
static void gtk_mdk_dialog_setbutton_callback(GtkMdkDialog *mdk_dialog,
                                              gint button_number,
                                              struct GtkMdkDialogRunInfo *runinfo)
{
  if (runinfo->close_id < 0)
    return;
  runinfo->button_number = button_number;
  gtk_mdk_dialog_shutdown_run(mdk_dialog, runinfo);
}

/*************************************************************************/
static gboolean gtk_mdk_dialog_quit_run(GtkMdkDialog *mdk_dialog,
                                        struct GtkMdkDialogRunInfo *runinfo)
{
  if (runinfo->close_id < 0)
    return FALSE;
  gtk_mdk_dialog_shutdown_run(mdk_dialog, runinfo);
  return FALSE;
}

/*************************************************************************/
static void gtk_mdk_dialog_mark_destroy(GtkMdkDialog *mdk_dialog,
                                        struct GtkMdkDialogRunInfo *runinfo)
{
  runinfo->destroyed = TRUE;
  if (runinfo->close_id < 0)
    return;
  else
    gtk_mdk_dialog_shutdown_run(mdk_dialog, runinfo);
}

/*************************************************************************/
static gint gtk_mdk_dialog_run_real(GtkMdkDialog* mdk_dialog,
                                    gboolean close_after)
{
  gboolean was_modal;
  struct GtkMdkDialogRunInfo ri = {-1,-1,-1,-1,FALSE};

  g_return_val_if_fail(mdk_dialog != NULL, -1);
  g_return_val_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog), -1);
  was_modal = GTK_WINDOW(mdk_dialog)->modal;
  if (!was_modal)
    gtk_window_set_modal(GTK_WINDOW(mdk_dialog), TRUE);

  /* There are several things that could happen to the dialog, and we
   * need to handle them all: click, delete_event, close, destroy */
  ri.clicked_id = gtk_signal_connect(GTK_OBJECT(mdk_dialog), "clicked",
                                     (GtkSignalFunc)gtk_mdk_dialog_setbutton_callback,
                                     &ri);
  ri.close_id = gtk_signal_connect(GTK_OBJECT(mdk_dialog), "close",
                                   (GtkSignalFunc)gtk_mdk_dialog_quit_run, &ri);
  ri.destroy_id = gtk_signal_connect(GTK_OBJECT(mdk_dialog), "destroy",
                                     (GtkSignalFunc)gtk_mdk_dialog_mark_destroy,
                                     &ri);
  if ( ! GTK_WIDGET_VISIBLE(GTK_WIDGET(mdk_dialog)) )
    gtk_widget_show(GTK_WIDGET(mdk_dialog));
  gtk_main();

  if (!ri.destroyed)
  {
    gtk_signal_disconnect(GTK_OBJECT(mdk_dialog), ri.destroy_id);
    if (!was_modal)
    {
      gtk_window_set_modal(GTK_WINDOW(mdk_dialog), FALSE);
    }
    if (ri.close_id >= 0) /* We didn't shut down the run? */
    {
      gtk_signal_disconnect(GTK_OBJECT(mdk_dialog), ri.close_id);
      gtk_signal_disconnect(GTK_OBJECT(mdk_dialog), ri.clicked_id);
    }
    if (close_after)
    {
      gtk_mdk_dialog_close(mdk_dialog);
    }
  }
  return ri.button_number;
}
/*
 * Blocks until the user clicks a button, or closes the dialog with the
 * window manager's close decoration (or by pressing Escape).
 *
 * You need to set up the dialog to do the right thing when a button
 * is clicked or delete_event is received; you must consider both of
 * those possibilities so that you know the status of the dialog when
 * gtk_mdk_dialog_run() returns. A common mistake is to forget about
 * Escape and the window manager close decoration; by default, these
 * call gtk_mdk_dialog_close(), which by default destroys the dialog. If
 * your button clicks do not destroy the dialog, you don't know
 * whether the dialog is destroyed when gtk_mdk_dialog_run()
 * returns. This is bad.
 *
 * So you should either close the dialog on button clicks as well, or
 * change the gtk_mdk_dialog_close() behavior to hide instead of
 * destroy. You can do this with gtk_mdk_dialog_close_hides().
 *
 * Return value:  If a button was pressed, the button number is returned, or
 * -1 if the dialog is closed by any other means.
 */
 gint gtk_mdk_dialog_run(GtkMdkDialog *mdk_dialog)
{
  return gtk_mdk_dialog_run_real(mdk_dialog,FALSE);
}

/*
 * See gtk_mdk_dialog_run(). The only difference is that this function calls
 * gtk_mdk_dialog_close() before returning, if the dialog was not already closed.
 *
 * Return value: If a button was pressed, the button number. Otherwise -1.
 */
gint gtk_mdk_dialog_run_and_close(GtkMdkDialog* mdk_dialog)
{
  return gtk_mdk_dialog_run_real(mdk_dialog,TRUE);
}

/*
 * Set the default button for the dialog.
 * The default button will be activated if the user just presses return.
 * Usually you should make the least-destructive button the default.
 * Otherwise, the most commonly-used button.
 */
void gtk_mdk_dialog_set_default(GtkMdkDialog *mdk_dialog, gint button)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(mdk_dialog->NbButtons>=button);

  gtk_widget_grab_default(mdk_dialog->Buttons[button]);
}

/*
 * This is a convenience function so you don't have to connect callbacks
 * to each button just to close the dialog.
 */
void gtk_mdk_dialog_close_on_click(GtkMdkDialog *mdk_dialog,
                                   gboolean close_on_click)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  mdk_dialog->close_on_click = close_on_click;
}

/*
 * Some dialogs are expensive to create, so you want to keep them around and just
 * gtk_widget_show() them when they are opened, and gtk_widget_hide() them when
 * they're closed. Other dialogs are expensive to keep around, so you want to
 * gtk_widget_destroy() them when they're closed. It's a judgment call you
 * will need to make for each dialog.
 */
void gtk_mdk_dialog_close_hides(GtkMdkDialog *mdk_dialog,
                                gboolean just_hide)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  mdk_dialog->just_hide = just_hide;
}


/* Set the sensitivity of a button. */
void gtk_mdk_dialog_set_sensitive(GtkMdkDialog *mdk_dialog,
                                  gint button,
                                  gboolean setting)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(mdk_dialog->NbButtons>=button);
  gtk_widget_set_sensitive(mdk_dialog->Buttons[button], setting);
}

/*
 * Simply gtk_signal_connect() to the "clicked" signal of the specified button.
 */
void gtk_mdk_dialog_button_connect(GtkMdkDialog *mdk_dialog,
                                   gint button,
                                   GtkSignalFunc callback,
                                   gpointer data)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(mdk_dialog->NbButtons>button);
  gtk_signal_connect(GTK_OBJECT(mdk_dialog->Buttons[button]), "clicked",
                     callback, data);
}

/*
 * gtk_signal_connect_object() to the "clicked" signal of the given button.
 */
void gtk_mdk_dialog_button_connect_object(GtkMdkDialog *mdk_dialog,
                                          gint button,
                                          GtkSignalFunc callback,
                                          GtkObject *obj)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(mdk_dialog->NbButtons>button);
  gtk_signal_connect_object(GTK_OBJECT(mdk_dialog->Buttons[button]), "clicked",
                            callback, obj);
}

/*************************************************************************/
void gtk_mdk_dialog_set_accelerator(GtkMdkDialog *mdk_dialog,
                                    gint button,
                                    const guchar accelerator_key,
                                    guint8 accelerator_mods)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  g_return_if_fail(mdk_dialog->NbButtons>button);
  gtk_widget_add_accelerator(GTK_WIDGET(mdk_dialog->Buttons[button]),
                             "clicked",
                             mdk_dialog->accelerators,
                             accelerator_key,
                             accelerator_mods,
                             GTK_ACCEL_VISIBLE);
}

/*************************************************************************/
static void gtk_mdk_dialog_button_clicked(GtkWidget *button,
                                          GtkWidget *dialog)
{
  gboolean close_on_click;
  int which;
  GtkMdkDialog *mdk_dialog;
  g_return_if_fail(dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(dialog));
  mdk_dialog = GTK_MDK_DIALOG(dialog);

  close_on_click = mdk_dialog->close_on_click;
  for (which = 0 ; which < mdk_dialog->NbButtons ; which++)
  {
    if (mdk_dialog->Buttons[which] == button)
    {
      gtk_signal_emit(GTK_OBJECT(dialog), gtk_mdk_dialog_signals[CLICKED],
                      which);
      break;
    }
  }
  /* The dialog may have been destroyed by the clicked
   * signal, which is why we had to save self_destruct.
   * Users should be careful not to set self_destruct
   * and then destroy the dialog themselves too. */
  if (close_on_click)
  {
    gtk_mdk_dialog_close(mdk_dialog);
  }
}

/*************************************************************************/
static gint gtk_mdk_dialog_key_pressed(GtkWidget *d, GdkEventKey *e)
{
  gboolean ret_val;
  g_return_val_if_fail(GTK_IS_MDK_DIALOG(d), TRUE);

  if (e->keyval == GDK_Escape)
  {
    gtk_mdk_dialog_close(GTK_MDK_DIALOG(d));
    return TRUE;
  }
  /* Have to call parent's handler, or the widget wouldn't get any
   *      key press events. Note that this is NOT done if the dialog
   *           may have been destroyed. */
  if (GTK_WIDGET_CLASS(mdk_dialog_parent_class)->key_press_event)
  {
    ret_val = (* (GTK_WIDGET_CLASS(mdk_dialog_parent_class)->key_press_event))(d, e);
  
    if ((GTK_WINDOW(d)->focus_widget == NULL) &&
        ( (e->keyval == GDK_Tab)   || (e->keyval == GDK_ISO_Left_Tab) ||
          (e->keyval == GDK_Up)    || (e->keyval == GDK_KP_Up) ||
          (e->keyval == GDK_Down)  || (e->keyval == GDK_KP_Down) ||
          (e->keyval == GDK_Left)  || (e->keyval == GDK_KP_Left) ||
          (e->keyval == GDK_Right) || (e->keyval == GDK_KP_Right) ))
    {
      switch (e->keyval)
      {
        case GDK_Tab: case GDK_ISO_Left_Tab:
          if (e->state & GDK_SHIFT_MASK)
            gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_TAB_BACKWARD);
          else
            gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_TAB_FORWARD);
          break;
        case GDK_Up: case GDK_KP_Up:
          gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_UP);
          break;
        case GDK_Down: case GDK_KP_Down:
          gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_DOWN);
          break;
        case GDK_Left: case GDK_KP_Left:
          gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_LEFT);
          break;
        case GDK_Right: case GDK_KP_Right:
          gtk_container_focus(GTK_CONTAINER(d), GTK_DIR_RIGHT);
          break;
      }
    }
    return ret_val;
  }
  else return FALSE; /* Not handled. */
}

/*************************************************************************/
static gint gtk_mdk_dialog_delete_event(GtkWidget *d, GdkEventAny *e)
{
  gtk_mdk_dialog_close(GTK_MDK_DIALOG(d));
  return TRUE; /* We handled it. */
}

/*************************************************************************/
static void gtk_mdk_dialog_destroy(GtkObject *dialog)
{
  GtkMdkDialog *mdk_dialog;
  g_return_if_fail(dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(dialog));
  mdk_dialog = GTK_MDK_DIALOG(dialog);
  
  if (mdk_dialog->title)
    g_free(mdk_dialog->title);
  if (mdk_dialog->text)
    g_free(mdk_dialog->text);
  if (GTK_OBJECT_CLASS(mdk_dialog_parent_class)->destroy)
    (* (GTK_OBJECT_CLASS(mdk_dialog_parent_class)->destroy))(dialog);
}

/*************************************************************************/
void gtk_mdk_dialog_close_real(GtkMdkDialog *mdk_dialog)
{
  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  gtk_widget_hide(GTK_WIDGET(mdk_dialog));
  if ( ! mdk_dialog->just_hide )
    gtk_widget_destroy(GTK_WIDGET(mdk_dialog));
}

/*************************************************************************/
void gtk_mdk_dialog_close(GtkMdkDialog *mdk_dialog)
{
  gint close_handled = FALSE;

  g_return_if_fail(mdk_dialog != NULL);
  g_return_if_fail(GTK_IS_MDK_DIALOG(mdk_dialog));
  gtk_signal_emit(GTK_OBJECT(mdk_dialog), gtk_mdk_dialog_signals[CLOSE],
                  &close_handled);
  if ( ! close_handled )
    gtk_mdk_dialog_close_real(mdk_dialog);
}

/*************************************************************************/
static void gtk_mdk_dialog_show(GtkWidget *d)
{
  if (GTK_WIDGET_CLASS(mdk_dialog_parent_class)->show)
    (* (GTK_WIDGET_CLASS(mdk_dialog_parent_class)->show))(d);
}
