/* gtkmdkfilesel.c
 *
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 *   Some code grabbed from xmms.
 */

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include <string.h>
#include <gtk/gtk.h>
#include "gtkmdkfilesel.h"

#include "gtkmdkintl.h"

static char *closed_folder[] =
{
  "16 16 8 1",
  "   c None",
  ".  c #909000",
  "+  c #000000",
  "@  c #EFE8EF",
  "#  c #FFF8CF",
  "$  c #FFF890",
  "%  c #CFC860",
  "&  c #FFC890",
  "                ",
  "  .....+        ",
  " .@##$$.+       ",
  ".%%%%%%%......  ",
  ".###########$%+ ",
  ".#$$$$$$$$$$&%+ ",
  ".#$$$$$$$&$&$%+ ",
  ".#$$$$$$$$&$&%+ ",
  ".#$$$$$&$&$&$%+ ",
  ".#$$$$$$&$&$&%+ ",
  ".#$$$&$&$&$&&%+ ",
  ".#&$&$&$&$&&&%+ ",
  ".%%%%%%%%%%%%%+ ",
  " ++++++++++++++ ",
  "                ",
  "                "
};

static char *open_folder[] =
{
  "16 16 12 1",
  "   c None",
  ".  c #808080",
  "+  c #E0E0D0",
  "@  c #4F484F",
  "#  c #909000",
  "$  c #FFF8EF",
  "%  c #CFC860",
  "&  c #003090",
  "*  c #7F7800",
  "=  c #FFC890",
  "-  c #FFF890",
  ";  c #2F3000",
  "        .       ",
  "       .+@      ",
  "   ###.$$+@     ",
  "  #%%.$$$$+@    ",
  "  #%.$$$&$$+@** ",
  "  #.+++&+&+++@* ",
  "############++@ ",
  "#$$$$$$$$$=%#++@",
  "#$-------=-=#+; ",
  " #---=--=-==%#; ",
  " #-----=-=-==#; ",
  " #-=--=-=-=-=#; ",
  "  #=-=-=-=-==#; ",
  "  ############; ",
  "   ;;;;;;;;;;;  ",
  "                "
};

typedef struct
{
  gchar   *path;
  gboolean visited;
} DirNode;

static void gtk_mdk_file_sel_class_init(GtkMdkFileSelClass *klass);
static void gtk_mdk_file_sel_init(GtkMdkFileSel *mdk_file_sel);
static void gtk_mdk_file_sel_show(GtkWidget *wid);

static void gtk_mdk_file_sel_expand_dir(GtkWidget *widget,
                                        GtkCTreeNode *parent_node);
static void gtk_mdk_file_sel_select_row_dir(GtkWidget *widget,
                                            GtkCTreeNode *node,
                                            gint column,
                                            GtkMdkFileSel *gtk_mdk_file_sel);
static void gtk_mdk_file_sel_destroy_dir_node_data(gpointer data);
static gint gtk_mdk_file_sel_dir_entry_compare_func(const void *a,
                                                    const void *b);
static gboolean gtk_mdk_file_sel_check_for_subdir(gchar *path);
static void gtk_mdk_file_sel_fill_file_clist(GtkWidget *widget,
                                             gchar *path);
static void gtk_mdk_file_sel_select_file(GtkWidget *clist,
                                         gint row,
                                         gint column,
                                         GdkEventButton *bevent,
                                         GtkMdkFileSel *gtk_mdk_file_sel);
static void gtk_mdk_file_sel_real_set_filename(GtkMdkFileSel *gtk_mdk_file_sel);
static void gtk_mdk_file_re_sort(GtkCList *clist, gint column, gpointer data);

static GtkWindowClass *gtk_mdk_file_sel_parent_class = NULL;

/*************************************************************************/
GtkType gtk_mdk_file_sel_get_type(void)
{
  static GtkType gtk_mdk_file_sel_type = 0;

  if (!gtk_mdk_file_sel_type)
    {
      static const GtkTypeInfo gtk_mdk_file_sel_info =
      {
        "GtkMdkFileSel",
        sizeof(GtkMdkFileSel),
        sizeof(GtkMdkFileSelClass),
        (GtkClassInitFunc)gtk_mdk_file_sel_class_init,
        (GtkObjectInitFunc)gtk_mdk_file_sel_init,
        /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      gtk_mdk_file_sel_type = gtk_type_unique(GTK_TYPE_WINDOW, &gtk_mdk_file_sel_info);
    }

  return gtk_mdk_file_sel_type;
}

/*************************************************************************/
static void gtk_mdk_file_sel_class_init(GtkMdkFileSelClass *klass)
{
  GtkWidgetClass *widget_class;
  
  widget_class = (GtkWidgetClass*) klass;
  gtk_mdk_file_sel_parent_class = gtk_type_class(GTK_TYPE_WINDOW);

  widget_class->show = gtk_mdk_file_sel_show;
}

/*************************************************************************/
static void gtk_mdk_file_sel_init(GtkMdkFileSel *gtk_mdk_file_sel)
{
  GtkWidget *separator, *Scr, *hbox, *vbox;
  GtkCTreeNode *root_node, *node;
  char *root_text = "/", *node_text="dummy";
  DirNode *dirnode;
  gchar *tmpdir;
  gchar *titles[] = { _("Name"), _("Size") };

  tmpdir = g_get_current_dir();
  gtk_mdk_file_sel->filename = g_strconcat(tmpdir, "/", NULL);
  g_free(tmpdir);
  gtk_mdk_file_sel->vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(gtk_mdk_file_sel), gtk_mdk_file_sel->vbox);
  gtk_widget_show(gtk_mdk_file_sel->vbox);


  /* The buttons at the bottom of the window */
  gtk_mdk_file_sel->button_box = gtk_hbutton_box_new();
  gtk_container_set_border_width(GTK_CONTAINER(gtk_mdk_file_sel->button_box),
                                 10);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_mdk_file_sel->button_box),
                            GTK_BUTTONBOX_END);
  gtk_box_pack_end(GTK_BOX(gtk_mdk_file_sel->vbox),
                   gtk_mdk_file_sel->button_box, FALSE, TRUE, 0);
  gtk_widget_show(gtk_mdk_file_sel->button_box);

  separator = gtk_hseparator_new();
  gtk_box_pack_end(GTK_BOX(gtk_mdk_file_sel->vbox), separator, FALSE, TRUE, 0);
  gtk_widget_show(separator);
  /* The 'OK' Button */
  gtk_mdk_file_sel->ok_button = gtk_button_new_with_label(_(" OK "));
  gtk_container_add(GTK_CONTAINER(gtk_mdk_file_sel->button_box),
                    gtk_mdk_file_sel->ok_button);
  gtk_widget_show(gtk_mdk_file_sel->ok_button);
  /* The 'Cancel' Button */
  gtk_mdk_file_sel->cancel_button = gtk_button_new_with_label(_(" Cancel "));
  gtk_container_add(GTK_CONTAINER(gtk_mdk_file_sel->button_box),
                    gtk_mdk_file_sel->cancel_button);
  gtk_widget_show(gtk_mdk_file_sel->cancel_button);

  /*Frame around dir tree and file list*/
  gtk_mdk_file_sel->frame = gtk_frame_new("");
  gtk_container_set_border_width(GTK_CONTAINER(gtk_mdk_file_sel->frame), 10);
  gtk_box_pack_start(GTK_BOX(gtk_mdk_file_sel->vbox), gtk_mdk_file_sel->frame, TRUE, TRUE, 0);
  gtk_widget_show(gtk_mdk_file_sel->frame);		
  
  /* The hbox that contains Dir tree and file list */
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(gtk_mdk_file_sel->frame), hbox);
  gtk_widget_show(hbox);

  /* The Directory tree */
  Scr = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(Scr),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_container_set_border_width(GTK_CONTAINER(Scr), 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), Scr);
  gtk_widget_show(Scr);

  gtk_widget_realize(GTK_WIDGET(gtk_mdk_file_sel));
  gtk_mdk_file_sel->closed_folder_pixmap = 
    gdk_pixmap_create_from_xpm_d(GTK_WIDGET(gtk_mdk_file_sel)->window,
                                 &gtk_mdk_file_sel->closed_folder_mask, NULL,
                                 closed_folder);
  gtk_mdk_file_sel->open_folder_pixmap =
    gdk_pixmap_create_from_xpm_d(GTK_WIDGET(gtk_mdk_file_sel)->window, 
                                 &gtk_mdk_file_sel->open_folder_mask, NULL,
                                 open_folder);

  gtk_mdk_file_sel->dir_ctree = gtk_ctree_new(1, 0);
  gtk_widget_set_usize(gtk_mdk_file_sel->dir_ctree, 250, 200);
  gtk_clist_set_column_auto_resize(GTK_CLIST(gtk_mdk_file_sel->dir_ctree),
                                   0, TRUE);
  gtk_clist_set_selection_mode(GTK_CLIST(gtk_mdk_file_sel->dir_ctree),
                               GTK_SELECTION_BROWSE);
  gtk_ctree_set_line_style(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                           GTK_CTREE_LINES_DOTTED);
  gtk_signal_connect_object(GTK_OBJECT(gtk_mdk_file_sel->dir_ctree),
                            "tree_expand",
                            (GtkSignalFunc)gtk_mdk_file_sel_expand_dir,
                            GTK_OBJECT(gtk_mdk_file_sel));  
  gtk_signal_connect(GTK_OBJECT(gtk_mdk_file_sel->dir_ctree),
                     "tree_select_row",
                     (GtkSignalFunc)gtk_mdk_file_sel_select_row_dir,
                     gtk_mdk_file_sel);
  gtk_container_add(GTK_CONTAINER(Scr), gtk_mdk_file_sel->dir_ctree);
/*  gtk_object_set_user_data(GTK_OBJECT(tree), (gpointer) handler);*/

  /* The file list */
  Scr = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(Scr),
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);
  gtk_container_set_border_width(GTK_CONTAINER(Scr), 5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), Scr);
  gtk_widget_show(Scr);
  gtk_mdk_file_sel->file_clist = gtk_clist_new_with_titles(2, titles);
  gtk_widget_set_usize(gtk_mdk_file_sel->file_clist, 250, 200);
  gtk_signal_connect(GTK_OBJECT(gtk_mdk_file_sel->file_clist), "select_row",
                     (GtkSignalFunc)gtk_mdk_file_sel_select_file,
                     (gpointer)gtk_mdk_file_sel);
  gtk_signal_connect(GTK_OBJECT(gtk_mdk_file_sel->file_clist),
		     "click_column",
                     (GtkSignalFunc)gtk_mdk_file_re_sort,
                     NULL);
  gtk_container_add(GTK_CONTAINER(Scr), gtk_mdk_file_sel->file_clist);
  gtk_clist_set_auto_sort(GTK_CLIST(gtk_mdk_file_sel->file_clist), TRUE);  
  gtk_widget_show(gtk_mdk_file_sel->file_clist);
  
  
  /* FileSelection Entry...*/
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_box_pack_end(GTK_BOX(gtk_mdk_file_sel->vbox), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  gtk_mdk_file_sel->file_entry = gtk_entry_new();
  gtk_signal_connect_object(GTK_OBJECT(gtk_mdk_file_sel->file_entry),
                            "focus_in_event",
                            (GtkSignalFunc)gtk_widget_grab_default,
                            GTK_OBJECT(gtk_mdk_file_sel->ok_button));
  gtk_signal_connect_object(GTK_OBJECT(gtk_mdk_file_sel->file_entry),
                            "activate",
                            (GtkSignalFunc)gtk_button_clicked,
                            GTK_OBJECT(gtk_mdk_file_sel->ok_button));
  gtk_box_pack_start_defaults(GTK_BOX(vbox), gtk_mdk_file_sel->file_entry);
  gtk_widget_show(gtk_mdk_file_sel->file_entry);





  
  
  
  root_node = gtk_ctree_insert_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                    NULL, NULL, &root_text, 4,
                                    gtk_mdk_file_sel->closed_folder_pixmap,
                                    gtk_mdk_file_sel->closed_folder_mask,
                                    gtk_mdk_file_sel->open_folder_pixmap,
                                    gtk_mdk_file_sel->open_folder_mask,
                                    FALSE, FALSE);
  dirnode = g_malloc(sizeof(DirNode));
  dirnode->path = g_strdup("/");
  dirnode->visited = FALSE;
  gtk_ctree_node_set_row_data_full(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                   root_node,
                                   dirnode,
                                   gtk_mdk_file_sel_destroy_dir_node_data);
  node = gtk_ctree_insert_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                               root_node, NULL, &node_text, 4,
                               NULL, NULL, NULL, NULL, TRUE, TRUE);
  gtk_ctree_expand(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), root_node);
  gtk_widget_show(gtk_mdk_file_sel->dir_ctree);
  gtk_clist_set_row_height(GTK_CLIST(gtk_mdk_file_sel->dir_ctree),
                           gtk_mdk_file_sel->dir_ctree->style->font->ascent +
                           gtk_mdk_file_sel->dir_ctree->style->font->descent +
                           1);
}

/*************************************************************************/
GtkWidget *gtk_mdk_file_sel_new(void)
{
  GtkMdkFileSel *gtk_mdk_file_sel;

  gtk_mdk_file_sel = gtk_type_new(GTK_TYPE_MDK_FILE_SEL);
  return GTK_WIDGET(gtk_mdk_file_sel);
}

/*************************************************************************/
static void gtk_mdk_file_sel_show(GtkWidget *wid)
{
  GtkMdkFileSel *file_sel;

  file_sel = GTK_MDK_FILE_SEL(wid);
  if (GTK_WIDGET_CLASS(gtk_mdk_file_sel_parent_class)->show)
    (* (GTK_WIDGET_CLASS(gtk_mdk_file_sel_parent_class)->show))(wid);
  if (file_sel->filename)
    gtk_mdk_file_sel_real_set_filename(file_sel);
}

/*************************************************************************/
static void gtk_mdk_file_sel_expand_dir(GtkWidget *widget,
                                        GtkCTreeNode *parent_node)
{
  DIR *dir;
  struct dirent *dirent;
  gchar *path, *text, *dummy = "dummy";
  struct stat statbuf;
  GtkCTreeNode *node, *sub_node;
  DirNode *parent_dirnode, *dirnode;
  gboolean has_subdir = FALSE;
  GtkMdkFileSel *gtk_mdk_file_sel = GTK_MDK_FILE_SEL(widget);

  parent_dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), parent_node);
  if (!parent_dirnode->visited)
  {
    gtk_clist_freeze(GTK_CLIST(gtk_mdk_file_sel->dir_ctree));
    node = gtk_ctree_find_by_row_data(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                      parent_node, NULL);
    gtk_ctree_remove_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), node);
    if ((dir = opendir(parent_dirnode->path)) != NULL)
    {
      while ((dirent = readdir(dir)) != NULL)
      {
        path = g_strconcat(parent_dirnode->path, dirent->d_name, NULL);
        if (stat(path, &statbuf) != -1 &&
            S_ISDIR(statbuf.st_mode)   &&
            dirent->d_name[0] != '.')
        {
          dirnode = g_malloc(sizeof(DirNode));
          dirnode->path = g_strconcat(path, "/", NULL);
          dirnode->visited = FALSE;
          text = dirent->d_name;
          if (gtk_mdk_file_sel_check_for_subdir(dirnode->path))
            has_subdir = TRUE;
          else
            has_subdir = FALSE;
          node = gtk_ctree_insert_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                       parent_node, NULL,
                                       &text, 4,
                                       gtk_mdk_file_sel->closed_folder_pixmap,
                                       gtk_mdk_file_sel->closed_folder_mask,
                                       gtk_mdk_file_sel->open_folder_pixmap,
                                       gtk_mdk_file_sel->open_folder_mask,
                                       !has_subdir, FALSE);
          gtk_ctree_node_set_row_data_full(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                           node, dirnode,
                                           gtk_mdk_file_sel_destroy_dir_node_data);
          if (has_subdir)
            sub_node = gtk_ctree_insert_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), node,
               NULL, &dummy, 4, NULL, NULL, NULL, NULL, FALSE, FALSE);
        }
        g_free(path);
      }
      closedir(dir);
      gtk_ctree_sort_node(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), parent_node);
    }
    gtk_clist_thaw(GTK_CLIST(gtk_mdk_file_sel->dir_ctree));
    parent_dirnode->visited = TRUE;
  }
}

/*************************************************************************/
static void gtk_mdk_file_sel_select_row_dir(GtkWidget *widget,
                                            GtkCTreeNode *node,
                                            gint column,
                                            GtkMdkFileSel *gtk_mdk_file_sel)
{
  DirNode *dirnode;

  gtk_mdk_file_sel->selected_dir_node = node;
  dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(widget), node);
  if (dirnode)
  {
	gtk_frame_set_label(GTK_FRAME(gtk_mdk_file_sel->frame), dirnode->path);
	gtk_mdk_file_sel_fill_file_clist(gtk_mdk_file_sel->file_clist,
                                     dirnode->path);
  }
}

/*************************************************************************/
static void gtk_mdk_file_sel_destroy_dir_node_data(gpointer data)
{
  DirNode *node = data;

  g_free(node->path);
  g_free(node);
}

/*************************************************************************/
static gint gtk_mdk_file_sel_dir_entry_compare_func(const void *a, const void *b)
{
  if (!a || !b || !((DirNode *) a)->path)
    return -1;
  return strcmp(((DirNode *) a)->path, (gchar *) b);
}

/*************************************************************************/
static gboolean gtk_mdk_file_sel_check_for_subdir(gchar * path)
{
  DIR *dir;
  struct dirent *dirent;
  struct stat statbuf;
  gchar *npath;

  if ((dir = opendir(path)) != NULL)
  {
    while ((dirent = readdir(dir)) != NULL)
    {
      if (dirent->d_name[0] != '.')
      {
        npath = g_strconcat(path, dirent->d_name, NULL);
        if (stat(npath, &statbuf) != -1 && S_ISDIR(statbuf.st_mode))
        {
          g_free(npath);
          closedir(dir);
          return TRUE;
        }
        g_free(npath);
      }
    }
    closedir(dir);
  }
  return FALSE;
}

/*************************************************************************/
static void gtk_mdk_file_sel_fill_file_clist(GtkWidget *widget,
                                             gchar *path)
{
  DIR *dir;
  struct dirent *dirent;
  struct stat statbuf;
  gchar *npath;
  GtkCList *clist;
  gchar *text[2];
  gint file_list_width = 1;

  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_CLIST(widget));
  clist = GTK_CLIST(widget);
  
  gtk_clist_freeze(clist);
  gtk_clist_clear(clist);
  gtk_clist_set_column_width(clist, 0, file_list_width);
  if ((dir = opendir(path)) != NULL)
  {
    while ((dirent = readdir(dir)) != NULL)
    {
      if (dirent->d_name[0] != '.')
      {
        npath = g_strconcat(path, dirent->d_name, NULL);
        if (stat(npath, &statbuf) != -1 && !S_ISDIR(statbuf.st_mode))
        {
          int row;
          int width = gdk_string_width(GTK_WIDGET(clist)->style->font,
                                       dirent->d_name);
                
          text[0] = dirent->d_name;
	  text[1] = g_strdup_printf("%d", (int)statbuf.st_size);
          row = gtk_clist_append(clist, text);
	  g_free(text[1]);
          gtk_clist_set_row_data_full(clist, row,
                                      g_strdup(dirent->d_name),
                                      (GtkDestroyNotify)g_free);
          if (width > file_list_width)
          {
            file_list_width = width;
            gtk_clist_set_column_width(clist, 0, file_list_width);
          }
        }
        g_free(npath);
      }
    }
    closedir(dir);
  }
  gtk_clist_thaw(clist);
}

/*************************************************************************/
static void gtk_mdk_file_sel_select_file(GtkWidget *clist,
                                         gint row,
                                         gint column,
                                         GdkEventButton *bevent,
                                         GtkMdkFileSel *gtk_mdk_file_sel)
{
  gchar *filename, *temp = NULL;

  g_return_if_fail(GTK_IS_CLIST(clist));
  g_return_if_fail(gtk_mdk_file_sel != NULL);
  g_return_if_fail(GTK_IS_MDK_FILE_SEL(gtk_mdk_file_sel));

  gtk_clist_get_text(GTK_CLIST(clist), row, 0, &temp);
  filename = g_strdup (temp);

  if (filename)
  {
    if (bevent && (bevent->type == GDK_2BUTTON_PRESS))
      gtk_button_clicked(GTK_BUTTON(gtk_mdk_file_sel->ok_button));
    else
      gtk_entry_set_text(GTK_ENTRY(gtk_mdk_file_sel->file_entry), filename);
    g_free (filename);
  }
}

/*************************************************************************/
static void gtk_mdk_file_sel_real_set_filename(GtkMdkFileSel *gtk_mdk_file_sel)
{
  gchar *CurrentPath;
  gint i;
  GtkCTreeNode *node, *nextnode;
  gboolean leaf;
  gchar *filename;
  GtkCList *clist = GTK_CLIST(gtk_mdk_file_sel->file_clist);

  gtk_ctree_collapse_recursive(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                               NULL);
  
  CurrentPath = g_malloc(strlen(gtk_mdk_file_sel->filename)+2);
  node = NULL;
  for (i=0 ; gtk_mdk_file_sel->filename[i] ; i++)
  {
    CurrentPath[i] = gtk_mdk_file_sel->filename[i];
    CurrentPath[i+1] = 0;

    if (CurrentPath[i] == '/')
    {
      nextnode = gtk_ctree_find_by_row_data_custom(
                                  GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                  node,
                                  CurrentPath,
                                  gtk_mdk_file_sel_dir_entry_compare_func);
      if (!nextnode)
        break;
      node = nextnode;
      gtk_ctree_get_node_info(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                              node, NULL, NULL, NULL, NULL, NULL, NULL, &leaf,
                              NULL);
      if (!leaf)
      {
        gtk_ctree_expand(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), node);
        gtk_ctree_select(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), node);
      }
    }
  }
  gtk_ctree_node_moveto(GTK_CTREE(gtk_mdk_file_sel->dir_ctree), node,
                        0, 0.5, 0.5);
  filename = strrchr(CurrentPath, '/');
  if (filename)
  {
    filename++;
    for (i=0 ; i<clist->rows ; i++)
    {
      if (!strcmp(filename, gtk_clist_get_row_data(clist, i)))
        gtk_clist_select_row(clist, i, 0);
    }
  }
  g_free(CurrentPath);
}

/*************************************************************************/
void gtk_mdk_file_sel_set_filename(GtkMdkFileSel *gtk_mdk_file_sel,
                                   const gchar   *filename)
{
  g_return_if_fail(gtk_mdk_file_sel != NULL);
  g_return_if_fail(GTK_IS_MDK_FILE_SEL(gtk_mdk_file_sel));
  g_return_if_fail(filename != NULL);

  if (gtk_mdk_file_sel->filename)
    g_free(gtk_mdk_file_sel->filename);
  gtk_mdk_file_sel->filename = g_strdup(filename);
  
  gtk_mdk_file_sel_real_set_filename(gtk_mdk_file_sel);
}

/*************************************************************************/
gchar *gtk_mdk_file_sel_get_filename(GtkMdkFileSel *gtk_mdk_file_sel)
{
  DirNode *dirnode;

  g_return_val_if_fail(gtk_mdk_file_sel != NULL, NULL);
  g_return_val_if_fail(GTK_IS_MDK_FILE_SEL(gtk_mdk_file_sel), NULL);

  dirnode = gtk_ctree_node_get_row_data(GTK_CTREE(gtk_mdk_file_sel->dir_ctree),
                                        gtk_mdk_file_sel->selected_dir_node);
  if (gtk_mdk_file_sel->filename)
    g_free(gtk_mdk_file_sel->filename);
  if (dirnode)
  {
    gtk_mdk_file_sel->filename = g_strconcat(dirnode->path,
                                             gtk_entry_get_text(GTK_ENTRY(gtk_mdk_file_sel->file_entry)),
                                             NULL);
  }
  return gtk_mdk_file_sel->filename;
}

/************************************************************************/
static gint gtk_mdk_file_sel_number_compare(GtkCList *clist,
                                            gconstpointer  ptr1,
                                            gconstpointer  ptr2)
{
  gint size1=0, size2=0;
  GtkCListRow *row1 = (GtkCListRow *) ptr1;
  GtkCListRow *row2 = (GtkCListRow *) ptr2;

  switch (row1->cell[clist->sort_column].type)
  {
    case GTK_CELL_TEXT:
      size1 = atoi(GTK_CELL_TEXT (row1->cell[clist->sort_column])->text);
      break;
    case GTK_CELL_PIXTEXT:
      size1 = atoi(GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text);
      break;
    default:
      break;
  }
  switch (row2->cell[clist->sort_column].type)
  {
    case GTK_CELL_TEXT:
      size2 = atoi(GTK_CELL_TEXT(row2->cell[clist->sort_column])->text);
      break;
    case GTK_CELL_PIXTEXT:
      size2 = atoi(GTK_CELL_PIXTEXT(row2->cell[clist->sort_column])->text);
      break;
    default:
      break;
  }
  return size2-size1;
}

/************************************************************************/
static void gtk_mdk_file_re_sort(GtkCList *clist, gint column, gpointer data)
{
  if (column == clist->sort_column)
  {
    if (clist->sort_type == GTK_SORT_ASCENDING)
      clist->sort_type = GTK_SORT_DESCENDING;
    else
      clist->sort_type = GTK_SORT_ASCENDING;
  } else
    gtk_clist_set_sort_column (clist, column);
  switch (column)
  {
    case 0: /* Sort by name */
      gtk_clist_set_compare_func(clist, NULL);
      break;
    case 1: /* Sort by size */
      gtk_clist_set_compare_func(clist, gtk_mdk_file_sel_number_compare);
      break;
  }
  gtk_clist_sort(clist);
}

