/* gtkmdkprogress.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkmdkprogress.h"

#include "gtkmdkintl.h"

static void gtk_mdk_progress_class_init(GtkMdkProgressClass *klass);
static void gtk_mdk_progress_init(GtkMdkProgress *mdk_progress);
static void gtk_mdk_progress_set_arg(GtkObject *object,
                                     GtkArg *arg, guint arg_id);
static void gtk_mdk_progress_get_arg(GtkObject *object,
                                     GtkArg *arg, guint arg_id);
static void gtk_mdk_progress_finalize(GtkObject *object);

static GtkWindowClass *mdk_progress_parent_class = NULL;

/*************************************************************************/
GtkType gtk_mdk_progress_get_type(void)
{
  static GtkType mdk_progress_type = 0;

  if (!mdk_progress_type)
  {
    static const GtkTypeInfo mdk_progress_info =
    {
      "GtkMdkProgress",
      sizeof(GtkMdkProgress),
      sizeof(GtkMdkProgressClass),
      (GtkClassInitFunc)gtk_mdk_progress_class_init,
      (GtkObjectInitFunc)gtk_mdk_progress_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
      (GtkClassInitFunc) NULL,
    };
    mdk_progress_type = gtk_type_unique(GTK_TYPE_WINDOW, &mdk_progress_info);
  }

  return mdk_progress_type;
}

/*************************************************************************/
static void gtk_mdk_progress_class_init(GtkMdkProgressClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  mdk_progress_parent_class = gtk_type_class(GTK_TYPE_WINDOW);

  object_class->set_arg = gtk_mdk_progress_set_arg;
  object_class->get_arg = gtk_mdk_progress_get_arg;
  object_class->finalize = gtk_mdk_progress_finalize;
}

/*************************************************************************/
static void gtk_mdk_progress_init(GtkMdkProgress *mdk_progress)
{
}

/*************************************************************************/
static void gtk_mdk_progress_set_arg(GtkObject *object,
                                     GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
static void gtk_mdk_progress_get_arg(GtkObject *object,
                                     GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
GtkWidget *gtk_mdk_progress_new(GtkWindow *parent_window, gchar *caption)
{
  GtkMdkProgress *mdk_progress;
  GtkWidget *Frame, *Frame2, *VBox, *Label;

  mdk_progress = gtk_type_new(GTK_TYPE_MDK_PROGRESS);

  gtk_widget_realize(GTK_WIDGET(mdk_progress));
  gdk_window_set_decorations(GTK_WIDGET(mdk_progress)->window, 0);
  gdk_window_set_functions(GTK_WIDGET(mdk_progress)->window, 0);

  gtk_window_set_position(GTK_WINDOW(mdk_progress), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(mdk_progress), GTK_WINDOW(parent_window));
  gtk_window_set_modal(GTK_WINDOW(mdk_progress), TRUE);
  
  Frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(Frame), GTK_SHADOW_OUT);
  gtk_container_add(GTK_CONTAINER(mdk_progress), Frame);
  gtk_widget_show(Frame);
  
  Frame2 = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(Frame2), GTK_SHADOW_IN);
  gtk_container_add(GTK_CONTAINER(Frame), Frame2);
  gtk_container_set_border_width(GTK_CONTAINER(Frame2), 5);
  gtk_widget_show(Frame2);
  
  VBox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(Frame2), VBox);
  gtk_container_set_border_width(GTK_CONTAINER(VBox), 5);
  gtk_widget_show(VBox);
  
  Label = gtk_label_new(caption);
  gtk_container_add(GTK_CONTAINER(VBox), Label);
  gtk_widget_show(Label);
  
  mdk_progress->adj = (GtkAdjustment*)gtk_adjustment_new(0, 0, 100, 0, 0, 0);
  mdk_progress->Bar = gtk_progress_bar_new_with_adjustment(mdk_progress->adj);
  gtk_container_add(GTK_CONTAINER(VBox), mdk_progress->Bar);
  gtk_progress_set_activity_mode(GTK_PROGRESS(mdk_progress->Bar), TRUE);
  gtk_progress_bar_set_activity_step(GTK_PROGRESS_BAR(mdk_progress->Bar), 5);
  gtk_progress_bar_set_activity_blocks(GTK_PROGRESS_BAR(mdk_progress->Bar), 3);
  gtk_widget_show(mdk_progress->Bar);

  return GTK_WIDGET(mdk_progress);
}

/*************************************************************************/
static void gtk_mdk_progress_finalize(GtkObject *object)
{
  GtkMdkProgress *mdk_progress;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_MDK_PROGRESS(object));

  mdk_progress = GTK_MDK_PROGRESS(object);

  (*GTK_OBJECT_CLASS(mdk_progress_parent_class)->finalize)(object);
}

/*************************************************************************/
void gtk_mdk_progress_update(GtkMdkProgress *mdk_progress)
{
  gfloat Value;

  Value = mdk_progress->adj->value + 5;
  if (Value > mdk_progress->adj->upper)
    Value = mdk_progress->adj->lower;
  gtk_progress_set_value(GTK_PROGRESS(mdk_progress->Bar), Value);
  while (gtk_events_pending())
    gtk_main_iteration();
}
