/* gtkmdktoolbar.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkmdktoolbar.h"

#include "gtkmdkintl.h"

static void gtk_mdk_toolbar_class_init(GtkMdkToolbarClass *klass);
static void gtk_mdk_toolbar_init(GtkMdkToolbar *mdk_toolbar);
static void gtk_mdk_toolbar_set_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id);
static void gtk_mdk_toolbar_get_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id);
static void gtk_mdk_toolbar_finalize(GtkObject *object);
static gboolean gtk_mdk_toolbar_button_press_event(GtkWidget      *toolbar,
                                                   GdkEventButton *event);

static GtkToolbarClass *mdk_toolbar_parent_class = NULL;

/*************************************************************************/
GtkType gtk_mdk_toolbar_get_type(void)
{
  static GtkType mdk_toolbar_type = 0;

  if (!mdk_toolbar_type)
  {
    static const GtkTypeInfo mdk_toolbar_info =
    {
      "GtkMdkToolbar",
      sizeof(GtkMdkToolbar),
      sizeof(GtkMdkToolbarClass),
      (GtkClassInitFunc)gtk_mdk_toolbar_class_init,
      (GtkObjectInitFunc)gtk_mdk_toolbar_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
      (GtkClassInitFunc) NULL,
    };
    mdk_toolbar_type = gtk_type_unique(GTK_TYPE_EVENT_BOX, &mdk_toolbar_info);
  }

  return mdk_toolbar_type;
}

/*************************************************************************/
static void gtk_mdk_toolbar_class_init(GtkMdkToolbarClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  mdk_toolbar_parent_class = gtk_type_class(GTK_TYPE_TOOLBAR);
  
  object_class->set_arg = gtk_mdk_toolbar_set_arg;
  object_class->get_arg = gtk_mdk_toolbar_get_arg;
  object_class->finalize = gtk_mdk_toolbar_finalize;
  widget_class->button_press_event = gtk_mdk_toolbar_button_press_event;
}

/*************************************************************************/
static void gtk_mdk_toolbar_init(GtkMdkToolbar *mdk_toolbar)
{

}

/*************************************************************************/
static void gtk_mdk_toolbar_set_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
static void gtk_mdk_toolbar_get_arg(GtkObject *object,
                                    GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
static gboolean gtk_mdk_toolbar_button_press_event(GtkWidget      *toolbar,
                                                   GdkEventButton *event)
{
  GtkMdkToolbar *mdk_toolbar = (GtkMdkToolbar *)toolbar;
  if (event->button == 3)
  {
    gtk_menu_popup(GTK_MENU(mdk_toolbar->menu), NULL, NULL, NULL, NULL,
                   event->button, event->time);
    
    return TRUE;
  }
  return FALSE;
}

static void gtk_mdk_toolbar_icon_only(GtkMdkToolbar *mdk_toolbar)
{
  gtk_toolbar_set_style(GTK_TOOLBAR(GTK_MDK_TOOLBAR(mdk_toolbar)->toolbar),
                        GTK_TOOLBAR_ICONS);
}

static void gtk_mdk_toolbar_text_only(GtkMdkToolbar *mdk_toolbar)
{
  gtk_toolbar_set_style(GTK_TOOLBAR(GTK_MDK_TOOLBAR(mdk_toolbar)->toolbar),
                        GTK_TOOLBAR_TEXT);
}

static void gtk_mdk_toolbar_both(GtkMdkToolbar *mdk_toolbar)
{
  gtk_toolbar_set_style(GTK_TOOLBAR(GTK_MDK_TOOLBAR(mdk_toolbar)->toolbar),
                        GTK_TOOLBAR_BOTH);
}

/*************************************************************************/
GtkWidget *gtk_mdk_toolbar_new(gchar *config_file)
{
  GtkMdkToolbar *mdk_toolbar;
  GtkWidget     *menu_item;

  mdk_toolbar = gtk_type_new(GTK_TYPE_MDK_TOOLBAR);
  mdk_toolbar->toolbar = GTK_TOOLBAR(gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
                                                     GTK_TOOLBAR_BOTH));

  gtk_toolbar_set_button_relief(GTK_TOOLBAR(mdk_toolbar->toolbar), GTK_RELIEF_NONE);

  gtk_container_add(GTK_CONTAINER(mdk_toolbar),
                    GTK_WIDGET(mdk_toolbar->toolbar));

  mdk_toolbar->menu = gtk_menu_new();
  menu_item = gtk_menu_item_new_with_label(_(" Icons only "));
  gtk_menu_append(GTK_MENU(mdk_toolbar->menu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
                            (GtkSignalFunc)gtk_mdk_toolbar_icon_only,
                            GTK_OBJECT(mdk_toolbar));
  gtk_widget_show(menu_item);
  menu_item = gtk_menu_item_new_with_label(_(" Text only "));
  gtk_menu_append(GTK_MENU(mdk_toolbar->menu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
                            (GtkSignalFunc)gtk_mdk_toolbar_text_only,
                            GTK_OBJECT(mdk_toolbar));
  gtk_widget_show(menu_item);
  menu_item = gtk_menu_item_new_with_label(_(" Both "));
  gtk_menu_append(GTK_MENU(mdk_toolbar->menu), menu_item);
  gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
                            (GtkSignalFunc)gtk_mdk_toolbar_both,
                            GTK_OBJECT(mdk_toolbar));
  gtk_widget_show(menu_item);
  
  gtk_widget_add_events(GTK_WIDGET(mdk_toolbar), GDK_BUTTON_PRESS_MASK);
  
  
  return GTK_WIDGET(mdk_toolbar);
}

/*************************************************************************/
static void gtk_mdk_toolbar_finalize(GtkObject *object)
{
  GtkMdkToolbar *mdk_toolbar;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_MDK_TOOLBAR(object));

  mdk_toolbar = GTK_MDK_TOOLBAR(object);

  (*GTK_OBJECT_CLASS(mdk_toolbar_parent_class)->finalize)(object);
}


