/* gtkwizard.c
 * 
 * This file is part of the MandrakeSoft Gtk Widgets Package
 * Copyright (C) 2000 DindinX <odin@mandrakesoft.com>
 * Copyright (C) 2000 MandrakeSoft
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "gtkmdkbutton.h"
#include "gtkwizard.h"

#include "gtkmdkintl.h"

enum
{
  PAGE_ENTER,
  PAGE_EXIT,
  LAST_SIGNAL
};

static void gtk_wizard_class_init(GtkWizardClass *klass);
static void gtk_wizard_init(GtkWizard *wizard);
static void gtk_wizard_set_arg(GtkObject *object,
                               GtkArg *arg, guint arg_id);
static void gtk_wizard_get_arg(GtkObject *object,
                               GtkArg *arg, guint arg_id);
static void gtk_wizard_finalize(GtkObject *object);
static void gtk_wizard_realize(GtkWidget *widget);
/*static void gtk_wizard_left_configure(GtkWidget *widget, GdkEvent *ev,
                                      GtkWizard *wizard);*/
static void gtk_wizard_title_configure(GtkWidget *widget,
                                       GdkEventConfigure *ev,
                                       GtkWizard *wizard);
static void gtk_wizard_page_changed(GtkWizard       *wizard,
                                    GtkNotebookPage *page,
                                    guint            page_num);

static GtkVBoxClass *wizard_parent_class = NULL;
static guint wizard_signals[LAST_SIGNAL] = { 0, 0 };

/*************************************************************************/
GtkType gtk_wizard_get_type(void)
{
  static GtkType wizard_type = 0;

  if (!wizard_type)
  {
    static const GtkTypeInfo wizard_info =
    {
      "GtkWizard",
      sizeof(GtkWizard),
      sizeof(GtkWizardClass),
      (GtkClassInitFunc)gtk_wizard_class_init,
      (GtkObjectInitFunc)gtk_wizard_init,
      /* reserved_1 */ NULL,
      /* reserved_2 */ NULL,
      (GtkClassInitFunc) NULL,
    };
    wizard_type = gtk_type_unique(GTK_TYPE_VBOX, &wizard_info);
  }

  return wizard_type;
}

/*************************************************************************/
static void gtk_wizard_class_init(GtkWizardClass *klass)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  object_class = (GtkObjectClass*) klass;
  widget_class = (GtkWidgetClass*) klass;

  wizard_parent_class = gtk_type_class(GTK_TYPE_VBOX);

  wizard_signals[PAGE_ENTER] = gtk_signal_new(
              "page_enter", GTK_RUN_FIRST, object_class->type,
              GTK_SIGNAL_OFFSET(GtkWizardClass, page_enter),
              gtk_marshal_NONE__UINT, GTK_TYPE_NONE, 1, GTK_TYPE_UINT);
  wizard_signals[PAGE_EXIT] = gtk_signal_new(
              "page_exit", GTK_RUN_FIRST, object_class->type,
              GTK_SIGNAL_OFFSET(GtkWizardClass, page_exit),
              gtk_marshal_NONE__UINT, GTK_TYPE_NONE, 1, GTK_TYPE_UINT);

  gtk_object_class_add_signals(object_class, wizard_signals, LAST_SIGNAL);

  object_class->set_arg = gtk_wizard_set_arg;
  object_class->get_arg = gtk_wizard_get_arg;
  object_class->finalize = gtk_wizard_finalize;
  widget_class->realize = gtk_wizard_realize;
}

/*************************************************************************/
static void gtk_wizard_init(GtkWizard *wizard)
{
  GtkWidget *hbox, *table, *hbbox;

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_end(GTK_BOX(wizard), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);
  
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 2);
  gtk_box_set_homogeneous(GTK_BOX(hbox), FALSE);

/*  hbbox = gtk_hbutton_box_new();
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbbox), 4);*/
  hbbox = gtk_hbox_new(FALSE, 4);
  gtk_box_pack_end(GTK_BOX(hbox), hbbox, FALSE, FALSE, 0);
  gtk_widget_show(hbbox);

  /* The help button */
  wizard->help_button = gtk_mdk_button_new(GTK_BUTTON_HELP);
  gtk_box_pack_start(GTK_BOX(hbbox), wizard->help_button, TRUE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(wizard->help_button, GTK_CAN_DEFAULT);
  gtk_widget_show(wizard->help_button);

  /* The back button */
  wizard->back_button = gtk_mdk_button_new(GTK_BUTTON_BACK);
  gtk_box_pack_start(GTK_BOX(hbbox), wizard->back_button, TRUE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(wizard->back_button, GTK_CAN_DEFAULT);
  gtk_widget_show(wizard->back_button);

  /* The next/finish button */
  wizard->next_button = gtk_mdk_button_new(GTK_BUTTON_NEXT);
  gtk_box_pack_start(GTK_BOX(hbbox), wizard->next_button, TRUE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(wizard->next_button, GTK_CAN_DEFAULT);
  gtk_widget_show(wizard->next_button);

  /* The Cancel button */
  wizard->cancel_button = gtk_mdk_button_new(GTK_BUTTON_CANCEL);
  gtk_box_pack_start(GTK_BOX(hbbox), wizard->cancel_button, TRUE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(wizard->cancel_button, GTK_CAN_DEFAULT);
  gtk_widget_show(wizard->cancel_button);

  table = gtk_table_new(2, 2, FALSE);
  gtk_box_pack_start_defaults(GTK_BOX(wizard), table);

  /* The Title */
  wizard->title = gtk_drawing_area_new();
  gtk_widget_set_usize(GTK_WIDGET(wizard->title), 540, 100);
  gtk_table_attach(GTK_TABLE(table), wizard->title, 0,2, 0,1,
                   GTK_FILL,
                   GTK_FILL,
                   0, 0);
  gtk_signal_connect(GTK_OBJECT(wizard->title), "configure_event",
                     (GtkSignalFunc)gtk_wizard_title_configure, wizard);
  wizard->top_pixmap = NULL;

  /* The Left part */
/*  wizard->left_pixmap = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(wizard->left_pixmap), 100, 300);
  gtk_table_attach(GTK_TABLE(table), wizard->left_pixmap, 0,1, 1,2,
                   GTK_FILL,
                   GTK_FILL,
                   0, 0);
  gtk_signal_connect(GTK_OBJECT(wizard->left_pixmap), "configure_event",
                     (GtkSignalFunc)gtk_wizard_left_configure, wizard);*/

  /* The Notebook */
  wizard->notebook = gtk_notebook_new();
  gtk_table_attach_defaults(GTK_TABLE(table), wizard->notebook, 0,2, 1,2);
  gtk_widget_set_usize(wizard->notebook, 440, 300);
  gtk_widget_show(wizard->notebook);
  gtk_signal_connect_object(GTK_OBJECT(wizard->notebook), "switch_page",
                            (GtkSignalFunc)gtk_wizard_page_changed,
                            GTK_OBJECT(wizard));
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(wizard->notebook), FALSE);
  gtk_notebook_set_show_border(GTK_NOTEBOOK(wizard->notebook), FALSE);
  
  gtk_signal_connect_object(GTK_OBJECT(wizard->next_button), "clicked",
                            (GtkSignalFunc)gtk_wizard_next_page,
                            GTK_OBJECT(wizard));
  gtk_signal_connect_object(GTK_OBJECT(wizard->back_button), "clicked",
                            (GtkSignalFunc)gtk_wizard_prev_page,
                            GTK_OBJECT(wizard));
  wizard->midnight_blue.red = 16896;
  wizard->midnight_blue.green = 20992;
  wizard->midnight_blue.blue = 33792;
  wizard->left_blue.red = 4096;
  wizard->left_blue.green = 10496;
  wizard->left_blue.blue = 23040;

  wizard->title_string = g_strdup(_("Wizard"));
  wizard->title_font = gdk_fontset_load(_("-*-helvetica-medium-r-*-*-30-*-*-*-*-*-*-*,*"));

  wizard->subtitle_string = g_strdup("");
  wizard->subtitle_font = gdk_fontset_load(_("-*-helvetica-medium-r-*-*-18-*-*-*-*-*-*-*,*"));

  wizard->nb_pages = 0;
}

/*************************************************************************/
static void gtk_wizard_set_arg(GtkObject *object,
                               GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
static void gtk_wizard_get_arg(GtkObject *object,
                               GtkArg *arg, guint arg_id)
{

}

/*************************************************************************/
GtkWidget *gtk_wizard_new(void)
{
  GtkWizard *wizard;

  wizard = gtk_type_new(GTK_TYPE_WIZARD);

  return GTK_WIDGET(wizard);
}

/*************************************************************************/
static void gtk_wizard_finalize(GtkObject *object)
{
  GtkWizard *wizard;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_WIZARD(object));

  wizard = GTK_WIZARD(object);

  (*GTK_OBJECT_CLASS(wizard_parent_class)->finalize)(object);
}

/*************************************************************************/
static void gtk_wizard_realize(GtkWidget *widget)
{
  GdkColormap *colormap;
  GtkWizard *wizard = (GtkWizard *)widget;

  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_WIZARD(widget));

  if (GTK_WIDGET_CLASS(wizard_parent_class)->realize)
    (*GTK_WIDGET_CLASS(wizard_parent_class)->realize) (widget);
  
  colormap = gdk_window_get_colormap(widget->window);
  gdk_color_alloc(colormap, &(wizard->midnight_blue));
  gdk_color_alloc(colormap, &(wizard->left_blue));
}

/*************************************************************************/
/*static void gtk_wizard_left_configure(GtkWidget *widget, GdkEvent *ev,
                                      GtkWizard *wizard)
{
  gdk_window_set_background(widget->window, &(wizard->left_blue));
}*/

/*************************************************************************/
static void gtk_wizard_title_configure(GtkWidget *widget,
                                       GdkEventConfigure *ev,
                                       GtkWizard *wizard)
{
  GdkPixmap *pixmap;
  GdkGC *gc;

  pixmap = gdk_pixmap_new(widget->window, ev->width, ev->height, -1);
  gc = gdk_gc_new(widget->window);
  gdk_gc_set_foreground(gc, &(wizard->midnight_blue));
  gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0,
                     ev->width, ev->height);
  if (wizard->top_pixmap)
    gdk_window_copy_area(pixmap, gc, 0,0,
                         wizard->top_pixmap, 0,0, 540,100);
/*  gdk_gc_set_foreground(gc, &(widget->style->white));*/
  
  gdk_draw_text(pixmap, wizard->title_font, widget->style->white_gc,
                (ev->width-gdk_string_width(wizard->title_font, 
                                            wizard->title_string))/2,
                50, wizard->title_string, strlen(wizard->title_string));
  gdk_draw_text(pixmap, wizard->subtitle_font, widget->style->white_gc,
                (ev->width-gdk_string_width(wizard->subtitle_font,
                                            wizard->subtitle_string))/2,
                80, wizard->subtitle_string, strlen(wizard->subtitle_string));

  gdk_window_set_back_pixmap(widget->window, pixmap, FALSE);
  gdk_window_clear(widget->window);
  gdk_gc_destroy(gc);
  gdk_pixmap_unref(pixmap);
}

/*************************************************************************/
void gtk_wizard_add_page(GtkWizard *wizard, GtkWidget *page)
{
  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));
  g_return_if_fail(page != NULL);

  gtk_notebook_append_page(GTK_NOTEBOOK(wizard->notebook), page,
                           gtk_label_new(" - "));
  wizard->nb_pages++;
}

/*************************************************************************/
void gtk_wizard_set_title(GtkWizard *wizard, gchar *str)
{
  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));
  g_return_if_fail(str != NULL);

  g_free(wizard->title_string);
  wizard->title_string = g_strdup(str);
  
  gtk_widget_queue_resize(wizard->title);
}

/*************************************************************************/
void gtk_wizard_set_subtitle(GtkWizard *wizard, gchar *str)
{
  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));
  g_return_if_fail(str != NULL);

  g_free(wizard->subtitle_string);
  wizard->subtitle_string = g_strdup(str);
  
  gtk_widget_queue_resize(wizard->title);
}

/*************************************************************************/
static void gtk_wizard_page_changed(GtkWizard       *wizard,
                                    GtkNotebookPage *page,
                                    guint            page_num)
{
  if (page_num == 0)
    gtk_widget_set_sensitive(wizard->back_button, FALSE);
  else
    gtk_widget_set_sensitive(wizard->back_button, TRUE);
}

/*************************************************************************/
void gtk_wizard_prev_page(GtkWizard *wizard)
{
  guint page;

  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));

  page = gtk_notebook_get_current_page(GTK_NOTEBOOK(wizard->notebook));
  if (page == 0)
    return;
  gtk_signal_emit(GTK_OBJECT(wizard), wizard_signals[PAGE_EXIT], page);
  gtk_notebook_prev_page(GTK_NOTEBOOK(wizard->notebook));
  page--;
  gtk_signal_emit(GTK_OBJECT(wizard), wizard_signals[PAGE_ENTER], page);
}

/*************************************************************************/
void gtk_wizard_next_page(GtkWizard *wizard)
{
  guint page;

  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));

  page = gtk_notebook_get_current_page(GTK_NOTEBOOK(wizard->notebook));
  if (page == wizard->nb_pages-1)
    return;
  gtk_signal_emit(GTK_OBJECT(wizard), wizard_signals[PAGE_EXIT], page);
  gtk_notebook_next_page(GTK_NOTEBOOK(wizard->notebook));
  page++;
  gtk_signal_emit(GTK_OBJECT(wizard), wizard_signals[PAGE_ENTER], page);
}

/*************************************************************************/
void gtk_wizard_set_top_pixmap_from_file(GtkWizard *wizard, gchar *filename)
{
  GdkBitmap *mask;
  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));

  wizard->top_pixmap = 
    gdk_pixmap_colormap_create_from_xpm(NULL,
                                        gdk_colormap_get_system(),
                                        &mask, NULL, filename);
}

/*************************************************************************/
void gtk_wizard_set_top_pixmap(GtkWizard *wizard, gchar **data)
{
  GdkBitmap *mask;
  g_return_if_fail(wizard != NULL);
  g_return_if_fail(GTK_IS_WIZARD(wizard));

  wizard->top_pixmap = 
    gdk_pixmap_colormap_create_from_xpm_d(NULL,
                                          gdk_colormap_get_system(),
                                          &mask, NULL, data);
}

